#############################################################
# RATFINK v0.8
# A library of RTF output utilities
# $Id: rtflib.tcl,v 1.9 1999/01/28 14:27:19 tobotras Exp $
# Created 4 Dec 1995 / Last updated $Date: 1999/01/28 14:27:19 $
#############################################################
#
# Copyright (C) 1996 Joe English
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#############################################################
# RATFINK lives at <URL:http://www.art.com/cost/ratfink/>
#############################################################

proc rtf:warning {text} {
    puts stderr "RTF warning: $text"
}

###
### Character formatting attributes
###

array set rtf:CharAttrs {
    Font	{table rtf:FontTable "f"}
    FontSize	{halfpts "fs"}
    Bold	{boolean "b"}
    Italic	{boolean "i"}
    Hidden	{boolean "v"}

    AllCaps	{boolean "caps"}
    SmallCaps	{boolean "scaps"}
    StrikeThru	{boolean "strike"}

    Underline {enum {
	{0 ulnone} {None ulnone}
	{1 ul} {Continuous ul} {Single ul}
	{Dot uld}
	{Double uldb}
	{Word ulw}
    }}

    Subscript		{flag "sub"}
    Superscript		{flag "super"}
    NoSuperSub		{flag "nosupersub"}
    ShiftDown		{halfpts "dn"}
    ShiftUp		{halfpts "up"}

    FGColor		{table rtf:ColorTable "cf"}
    BGColor		{table rtf:ColorTable "cb"}

    LetterSpace		{dimension "expndtw"}
    KernAbove		{halfpts "kerning"}

    TextDirection {enum {
	{RTL	"rtlch"}
	{LTR	"ltrch"}
    }}
    Charset	{table rtf:CharsetTable  "cchs"}
    Language	{table rtf:LanguageTable "lang"}
}

###
### Paragraph formatting attributes
###
array set rtf:ParaAttrs {
    TabStops    	{proc rtf:ExpandTabStops}
    FirstIndent 	{dimension "fi"}
    LeftIndent  	{dimension "li"}
    RightIndent 	{dimension "ri"}
    SpaceBefore 	{dimension "sb"}
    SpaceAfter  	{dimension "sa"}
    LineSpacing 	{dimension "sl"}
    KeepTogether	{flag "keep"}
    KeepWithNext	{flag "keepn"}
    PageBreakBefore	{flag "pagebb"}
    Hyphenate   	{boolean "hyphpar"}
    Quadding {enum {
	    {Left	"ql"}
	    {Right	"qr"}
	    {Justify	"qj"}
	    {Center	"qc"}
    }}
}
# All character attributes are also paragraph attributes:
array set rtf:ParaAttrs [array get rtf:CharAttrs]

# Add attributes for rules:
array set rtf:ParaAttrs {
    InnerBorders	{flag "brdrbtw"}
    Box 		{proc rtf:ExpandRuleStyle "box"}
    TopBorder		{proc rtf:ExpandRuleStyle "brdrt"}
    BottomBorder	{proc rtf:ExpandRuleStyle "brdrb"}
    LeftBorder		{proc rtf:ExpandRuleStyle "brdrl"}
    RightBorder		{proc rtf:ExpandRuleStyle "brdrr"}
    OutsideBorder	{proc rtf:ExpandRuleStyle "brdrbar"}
}

###
### Section formatting attributes
###
array set rtf:SectAttrs {
    PageWidth    	{dimension "pgwsxn"}
    PageHeight    	{dimension "pghsxn"}
    LeftMargin    	{dimension "marglsxn"}
    RightMargin    	{dimension "margrsxn"}
    TopMargin    	{dimension "margtsxn"}
    BottomMargin	{dimension "margbsxn"}
    GutterWidth        	{dimension "guttersxn"}
    Landscape		{flag "lndscpsxn"}
    HasTitlePage	{flag "titlepg"}
    HeaderPosition	{dimension "headery"}
    FooterPosition	{dimension "footery"}
    SectionBreak {enum {
	    {None	"sbknone"}
	    {Page	"sbkpage"}
	    {EvenPage	"sbkeven"}
	    {OddPage	"sbkodd"}
    }}
    VAlign {enum {
	    {Top	"vertalt"}
	    {Bottom	"vertalb"}
	    {Middle	"vertalc"} {Center	"vertalc"}
	    {Justify	"vertalj"}
    }}
    FirstPageNumber 	{integer starts}
    RestartPageNumbers {enum {
	{0 "pgncont"}
    	{1 "pgnrestart"}
    }}
    PageNumbering {enum {
	{Arabic 	"pgndec"} {Decimal "pgndec"}
	{UCRoman	"pgnucrm"}
	{LCRoman	"pgnlcrm"}
	{UCAlpha	"pgnucltr"}
	{LCAlpha	"pgnlcltr"}
    }}

    Columns		{integer "cols"}
    ColumnSepSpace	{dimension "colsx"}
    ColumnLine		{flag "linebetcol"}
    ColumnDirection	{enum {
	{RTL "rtlsect"}
	{LTR "ltrsect"}
    }}
}

###
### Style attribute handling
###

proc rtf:ExpandAttributes {tblName attlist} {
    upvar #0 $tblName table
    set result ""
    while {[llength $attlist] >= 2} {
	set param [lindex $attlist 0]
	set value [lindex $attlist 1]
	set attlist [lrange $attlist 2 end]

	if ![info exists table($param)] {
	    rtf:warning "No parameter $param in $tblName"
	    continue
	}

	set key $table($param)

	switch [lindex $key 0] {
	    unknown {
		rtf:warning "Don't grok parameter $param=$value in $tblName"
		continue
	    }

	    boolean {
		set csname [lindex $key 1]
		if $value {
		    set cs "\\$csname"
		} else {
		    set cs "\\${csname}0"
		}
	    }

	    flag {
		set csname [lindex $key 1]
		if $value {
		    set cs "\\$csname"
		} else {
		    rtf:warning "Parameter $param can only be turned on"
		}
	    }

	    integer {
		if ![regexp -- {^-?[0-9]+$} $value] {
		    rtf:warning "bad integer $value in parameter $param"
		    continue
		}
		set csname [lindex $key 1]
		set cs "\\${csname}$value"
	    }

	    dimension {
		set csname [lindex $key 1]
		set cs "\\${csname}[rtf:cvTwips $value]"
	    }

	    halfpts {
		set csname [lindex $key 1]
		set cs "\\${csname}[rtf:cvHalfpts $value]"
	    }

	    enum {
		catch {unset cs}
		set enumnames {}
		foreach enumpair [lindex $key 1] {
		    set enumname [lindex $enumpair 0]
		    if [string match $value $enumname] {
			set cs "\\[lindex $enumpair 1]";
			break;
		    }
		    lappend enumnames $enumname;
		}
		if ![info exists cs] {
		    rtf:warning \
		"Unknown value $param $value: must be one of $enumnames"
		    continue
		}
	    }

	    table {
		set subtblName [lindex $key 1]
		upvar #0 $subtblName subtable
		if ![info exists subtable($value)] {
		    rtf:warning "$param: no mapping for $value in $subtblName"
		    continue
		}
		if {[llength $key] == 3} {
		    set cs "\\[lindex $key 2]$subtable($value)"
		} else {
		    set cs $subtable($value)
		}
	    }

	    proc {
		set procName [lindex $key 1]
		set cs [$procName $value]
		if {[llength $key] == 3} {
		    set cs "\\[lindex $key 2]$cs"
		}
	    }

	    pseudo {
		# pseudo-argument; ignore
		set cs ""
	    }

	    default {
		rtf:warning "Bad parameter type [lindex $key 0] for $param"
		continue
	    }
	}
	append result $cs
    }
    if {[llength $attlist] != 0} {
	rtf:warning "Leftover arguments: <$attlist>"
    }
    return $result
}

# Expand attributes from associative array.
# 'paramlist' if present specifies the desired output order.
#
proc rtf:ExpandArray {tblName specsName {paramlist {}}} {
    upvar #0 $tblName table
    upvar 1 $specsName specs

    set speclist {}
    if {$paramlist == {}} { set paramlist [array names specs] }
    foreach param $paramlist {
	if [info exists specs($param)] {
	    lappend speclist $param $specs($param)
	}
    }
    # error-check:
    foreach param [array names specs] {
	if ![info exists table($param)] {
	    rtf:warning "No parameter $param in $tblName"
	}
    }

    return [rtf:ExpandAttributes $tblName $speclist]
}

###
### Stylesheet management
###

global rtf_styleSheet; 	# key: $styleName,(DESC|NUM|DEF|TYPE|BASEDON)
global rtf_styleNames;	set rtf_styleNames {}
global rtf_styleNum; 	set rtf_styleNum 1

proc rtf:paraStyle {name desc args} { rtf:defineStyle PARA $name $desc $args }
proc rtf:charStyle {name desc args} { rtf:defineStyle CHAR $name $desc $args }
proc rtf:sectStyle {name desc args} { rtf:defineStyle SECT $name $desc $args }

proc rtf:defineStyle {type name desc arglist} {
    global rtf_styleNum rtf_styleSheet rtf_styleNames

    set rtf_styleSheet($name,NUM) $rtf_styleNum
    set rtf_styleSheet($name,DESC) $desc
    set rtf_styleSheet($name,TYPE) $type
    set styledef ""

    incr rtf_styleNum
    lappend rtf_styleNames $name

    while {[string match "-*" [lindex $arglist 0]]} {
	set option [lindex $arglist 0]
	set value [lindex $arglist 1]
	set arglist [lrange $arglist 2 end]
	switch -- $option {
	    -basedon {
		if ![info exists rtf_styleSheet($value,DEF)] {
		    rtf:warning "$name: style $value not defined"
		    continue
		}
		if {$rtf_styleSheet($value,TYPE) != $type} {
		    rtf:warning "$name based on style of different type"
		    continue
		}
		set rtf_styleSheet($name,BASEDON) $value
		append styledef "$rtf_styleSheet($value,DEF)"
	    }
	    default {
		rtf:warning "rtf:defineStyle -- bad option $option"
	    }
	}
    }
    set attlist [lindex $arglist 0]
    switch $type {
	PARA { set styletable rtf:ParaAttrs }
	CHAR { set styletable rtf:CharAttrs }
	SECT { set styletable rtf:SectAttrs }
    }
    set stylespec [rtf:ExpandAttributes $styletable $attlist]
    append styledef $stylespec

    set rtf_styleSheet($name,DEF) $styledef
    return
}

proc rtf:writeStyleSheet {} {
    global rtf_styleNames rtf_styleSheet
    rtf:bgroup
    rtf:write "\\stylesheet"
    foreach style $rtf_styleNames {
	rtf:bgroup
	switch $rtf_styleSheet($style,TYPE) {
	    PARA { set styletype "\\s" }
	    CHAR { set styletype "\\cs" }
	    SECT { set styletype "\\ds" }
	}
	rtf:write "$styletype$rtf_styleSheet($style,NUM)"
	rtf:write "$rtf_styleSheet($style,DEF)"
	if [info exists rtf_styleSheet($style,BASEDON)] {
	    set basedon $rtf_styleSheet($style,BASEDON)
	    rtf:write "\\sbasedon$rtf_styleSheet($basedon,NUM)"
	}
	rtf:write " $rtf_styleSheet($style,DESC);"
	rtf:egroup
    }
    rtf:egroup
}

###
### Dimensions
###
global rtf_twipsMap
array set rtf_twipsMap "
	twip	1
	pt	20
	halfpt	10
	in	[expr 72 * 20]
	pc	[expr 12 * 20]
	pica	[expr 12 * 20]
	cm	567
	mm	56.7
"

# %%% The conversion factor 56.7 for 'mm' was determined --
# %%% by experimentation -- so RATFINK's definition of A4 paper (210mm x 297mm)
# %%% matches WFW95v7's definition (\paperw11907\paperh16840)
# %%% after rounding.  [expr (72 * 20) / 25.4] == 56.6929
# %%% is a *little* bit smaller than what Word appears to use.

proc rtf:cvTwips {dim} {
    global rtf_twipsMap
    if ![regexp -- {^(-?[0-9\.]+)([a-z]+)$} $dim dummy qty units] {
	rtf:warning "Bad dimension $dim"
	return 0
    }
    if ![info exists rtf_twipsMap($units)] {
	rtf:warning "Bad units $units"
	return 0
    }
    set scale $rtf_twipsMap($units)
    return [expr round($qty * $scale)]
}

proc rtf:cvHalfpts {dim} {
    if [regexp -- {^-?([0-9\.]+)$} $dim] {
	return $dim
    }
    return [expr round([rtf:cvTwips $dim] / 10)]
}

###
### Fonts
###
global rtf_fonttbl
set rtf_fonttbl [join {
	{{\f0\froman\fcharset[rtf:CharSet]\fprq2 Times New Roman;}}
	{{\f1\fswiss\fcharset[rtf:CharSet]\fprq2 Arial;}}
	{{\f2\fmodern\fcharset[rtf:CharSet]\fprq1 Courier New;}}
    } "" ]
array set rtf:FontTable {
	roman	0
	sans	1
	mono	2
}

proc rtf:writeFontTable {} {
    #+%%%
    global rtf_fonttbl;
    rtf:bgroup
    rtf:write "\\fonttbl [subst -nobackslashes -novariables $rtf_fonttbl]"
    rtf:egroup
}

###
### Tab stops
###

# The order of the control words must be <kind>? <leaders>? <position>,
array set rtf_TabAttrs {
    Align {enum {
	{Left	"tql"}
	{Right	"tqr"}
	{Center	"tqc"}
	{Decimal "tqdec"}
    }}
    Leaders {enum {
	{Dot	"tldot"}
	{Hyphen	"tlhyph"}
	{Under	"tlul"}
	{Thick	"tlth"}
	{Equal	"tleq"}
    }}
    Position {dimension "tx"}
}
proc rtf:tabStops {name stops} {
    global rtf_TabTable
    if [info exists rtf_TabTable($name)] {
	rtf:warning "Redefining tab settings $name"
    }
    set rtf_TabTable($name) [rtf:ExpandTabStops $stops]
}

proc rtf:ExpandTabStops {stops} {
    global rtf_TabAttrs rtf_TabTable
    set result ""
    if [regexp {^[a-zA-Z][-_a-zA-Z0-9]*$} $stops] {
	if [info exists rtf_TabTable($stops)] {
	    return $rtf_TabTable($stops)
	} else {
	    rtf:warning "Tab settings <$stops> not defined"
	    return ""
	}
    }
    foreach stop $stops {
	catch {unset tmp}
	set tmp(Position) [lindex $stop 0]
	array set tmp [lrange $stop 1 end];
	append result \
		[rtf:ExpandArray rtf_TabAttrs tmp "Align Leaders Position"]
    }
    return $result
}

###
### Rule styles
###

# Order of control words is:
# <brdrk>, <width>?, <space>?, <color>?
# <brdrk> == 'Style' is mandatory and must be first.
#
array set rtf_RuleAttrs {
    Style {enum {
	{Normal	"brdrs"}
	{Thick	"brdrth"}
	{Double	"brdrdb"}
	{Shadow	"brdrsh"}
	{Dot	"brdrdot"}
	{Dash	"brdrdash"}
	{Hairline	"brdrhair"}
    }}
    Margin	{dimension "brsp"}
    Thickness	{dimension "brdrw"}
    Color	{table rtf:colorTable "brdrcf"}
}
proc rtf:ruleStyle {name attlist} {
    global rtf_RuleTable
    if [info exists rtf_RuleTable($name)] {
	rtf:warning "Redefining rule style $name"
    }
    set rtf_RuleTable($name) [rtf:ExpandRuleStyle $attlist]
}
proc rtf:ExpandRuleStyle {attlist} {
    global rtf_RuleAttrs rtf_RuleTable
    if {[llength $attlist] == 1} {
	global rtf_RuleTable
	if [info exists rtf_RuleTable($attlist)] {
	    return $rtf_RuleTable($attlist)
	} else {
	    rtf:warning "No rule style <$attlist> defined"
	    return ""
	}
    }
    set tmp(Style) Normal;	# 'Style' is mandatory; set default
    array set tmp $attlist
    return [rtf:ExpandArray rtf_RuleAttrs tmp "Style Margin Thickness Color"]
}
# Pre-defined rule styles:
rtf:ruleStyle thin	{ Style Normal  Thickness 0.75pt }
rtf:ruleStyle thick 	{ Style Normal  Thickness 1.50pt }
rtf:ruleStyle double 	{ Style Double  Thickness 0.75pt }

###
### Document-wide attributes
###
array set rtf:DocAttrs {

    PaperWidth		{dimension "paperw"}
    PaperHeight		{dimension "paperh"}
    LeftMargin		{dimension "margl"}
    RightMargin		{dimension "margr"}
    TopMargin		{dimension "margt"}
    BottomMargin	{dimension "margb"}
    DefaultTabWidth	{dimension "deftab"}

    TwoSide		{flag "facingp"}
    MirrorMargins	{flag "margmirror"}
    Landscape		{flag "landscape"}
    FirstPageNumber	{integer "pgnstart"}
    WidowControl	{flag "widowctrl"}
    GutterWidth		{dimension "gutter"}

    SaveAsRTF			{flag "defformat"}
    Protection	{enum {
    	{Forms   	"formprot"}
    	{Revisions	"revisions\\revprot"}
    	{Annotations	"annotprot"}
    	{AllProtected	"allprot"}
    }}

    Hyphenate			{boolean "hyphauto"}
    HyphenationHotZone		{dimension "hyphhotz"}
    HyphenationLadderCount	{integer "hyphconsec"}
    HyphenateAllCaps		{boolean "hyphcaps"}

    FootnoteNumbering {enum {
    	{Arabic 	"ftnnar"}
    	{LCAlpha	"ftnnalc"}
    	{UCAlpha	"ftnnauc"}
    	{LCRoman	"ftnnrlc"}
    	{UCRoman	"ftnnruc"}
    	{Chicago	"ftnnchi"}
    }}
    FootnoteLocation {enum {
	{EndOfSection	"endnotes"}
	{EndOfDocument	"enddoc"}
    }}
    FootnotePlacement {enum {
    	{PageBottom	"ftnbj"}
    	{BeneathText	"ftntj"}
    }}
    FirstFootnoteNumber	{integer "ftnstart"}
    FootnoteRestart {enum {
	{Continuous	"ftnrstcont"}
	{AtSection	"ftnrestart"}
    	{AtPage		"ftnrstpg"}
    }}
}
# Add 'papersize' document options:
set tmp {}; foreach psz {
	{ A4    	210mm	297mm	}
	{ A5    	148mm	210mm	}
	{ B5    	176mm	250mm	}
	{ Letter	8.5in	11in 	}
	{ Legal 	8.5in	14in 	}
	{ Executive	7.25in	10.5in	}
} {
    set pszn [lindex $psz 0]
    set pszw [rtf:cvTwips [lindex $psz 1]]
    set pszh [rtf:cvTwips [lindex $psz 2]]
    lappend tmp [list $pszn "paperw$pszw\\paperh$pszh"]
}
set rtf:DocAttrs(PaperSize) "enum { $tmp }"

proc rtf:documentFormat {params} {
    global rtf_DocumentFormat
    array set rtf_DocumentFormat $params
}
proc rtf:writeDocumentFormat {} {
    global rtf_DocumentFormat
    if [info exists rtf_DocumentFormat] {
	rtf:write [rtf:ExpandArray rtf:DocAttrs rtf_DocumentFormat]
	rtf:write " "
    }
}

###
### Information group
###
proc rtf:writeInformationGroup {} {
    # %%% fake it for now...
    rtf:bgroup
    rtf:write "\\info"
	rtf:bgroup
	rtf:write "\\doccomm Created by RATFINK/Cost"
	rtf:egroup
    rtf:egroup
}

###
### Output
###
global rtf_fp
set rtf_fp stdout

proc rtf:write {text} {
    global rtf_fp
    puts $rtf_fp $text nonewline
}
proc rtf:bgroup {} {
    rtf:write \{
}
proc rtf:egroup {} {
    rtf:write \}
}

if { [array exists COST] } {
    substitution rtf:Escape {
	"{"	"\\{"
	"}"	"\\}"
 	"\\"	"\\\\"
	"\t"	" "
	"\n"	" "
    }
    substitution rtf:EscapeLineSpecific {
	"{"	"\\{"
	"}"	"\\}"
	"\\"	"\\\\"
	"\t"	"\\tab "
	"\n"	"\\line\n"
    }
} else {
    proc rtf:Escape {text} {
	regsub -all {[{}\\]} $text {\\&} text
	return $text
    }
}

proc rtf:text {text} {
    global rtf_state
    if !$rtf_state(inpara) {rtf:startPara [rtf:currentParaStyle]}
    rtf:write [rtf:Escape $text]
}
proc rtf:insert {data} {
    global rtf_state
    if !$rtf_state(inpara) {rtf:startPara [rtf:currentParaStyle]}
    rtf:write $data
}

# may be overridden by applications:
proc rtf:currentParaStyle {} {
    rtf:warning "No current paragraph style defined"
    return "-"
}

array set rtfSpecial {
    Tab			"\\tab "
    LineBreak		"\\line "
    PageBreak		"\\page "
    ColumnBreak		"\\column "

    EmDash		"\\emdash "
    EnDash		"\\endash "
    EmSpace		"{\\emspace  }"
    EnSpace		"{\\enspace  }"
    Bullet		"\\bullet "

    LSQuote		"\\lquote "
    RSQuote		"\\rquote "
    LDQuote		"{\\ldblquote}"
    RDQuote		"{\\rdblquote}"

    PageNumber		"\\chpgn "
    SectionNumber	"\\sectnum "
    FootnoteNumber	"\\chftn "
}
#    Ellipsis		"{\\expndtw20 ...}" ???
# There is reportedly an ellipsis character in the Symbol character set...
#

proc rtf:special {code} {
    global rtfSpecial
    if [info exists rtfSpecial($code)] {
	rtf:insert $rtfSpecial($code)
    } else {
	rtf:warning "Unrecognized special <$code>"
    }
}

proc rtf:tab {} 	{ rtf:write "\\tab " }
proc rtf:lineBreak {} 	{ rtf:write "\\line " }
proc rtf:pageBreak {} 	{ rtf:write "\\page " }
proc rtf:columnBreak {}	{ rtf:write "\\column " }

###
### Document structure
###

global rtf_state
array set rtf_state {
    inpara	0
    insection	0
    intable	0
    inrow	0
    incell	0
    cellno	0
    diversion	""
}

proc rtf:start {} {
    rtf:bgroup
# Header:
    rtf:write "\\rtf1\\ansi"
    rtf:write "\\deff0"
    rtf:writeFontTable
    #- fileTable
    #- colorTable
    rtf:writeStyleSheet
    #- revisionTable
# Document:
    rtf:writeInformationGroup
    rtf:writeDocumentFormat
# section text follows...
}
proc rtf:end {} {
    rtf:egroup
}

proc rtf:startSection {{style -}} {
    global rtf_state rtf_styleSheet
    rtf:endSection
    rtf:write "\\sectd"
    if {$style != "-"} {
	if {![info exists rtf_styleSheet($style,DEF)] \
		|| $rtf_styleSheet($style,TYPE) != "SECT"} {
	    rtf:warning "Style <$style> not a section style"
	} else {
	    rtf:write $rtf_styleSheet($style,DEF)
	}
	rtf:write " "
    }
    set rtf_state(insection) 1
}
proc rtf:endSection {} {
    global rtf_state
    if $rtf_state(insection) {
	rtf:write "\\sect "
    }
    set rtf_state(insection) 0
}

proc rtf:startPara {style} {
    global rtf_styleSheet rtf_state
    if $rtf_state(inpara) { rtf:write "\\par" }
    rtf:write "\\pard\\plain "
    if $rtf_state(intable) { rtf:write "\\intbl " }
    if [info exists rtf_styleSheet($style,DEF)] {
	if {$rtf_styleSheet($style,TYPE) != "PARA"} {
	    rtf:warning "$style not a paragraph style"
       } else { 
#
# I do not understand why we need the following line. Without it in Word 9x
# we'll see all text in a plain style. -A.V.
#
           rtf:write "$rtf_styleSheet($style,DEF)"
#
# If we put previous line *after* \s<number>, we'll have non-style formatting
# in rtf. No fun! -A.V. 
#          
           rtf:write "\\s$rtf_styleSheet($style,NUM) "
	}
    } elseif {$style != "-"} {
	rtf:warning "No definition for <$style> style"
    }
    set rtf_state(inpara) 1
}
proc rtf:endPara {} {
    global rtf_state
    if $rtf_state(inpara) {
	rtf:write "\\par\n"
    }
    set rtf_state(inpara) 0
}

proc rtf:startPhrase {style} {
    rtf:bgroup
    rtf:setCharStyle $style
}
proc rtf:endPhrase {} {
    rtf:egroup
}

proc rtf:setCharStyle {style} {
    global rtf_styleSheet rtf_state
    if !$rtf_state(inpara) { rtf:startPara [rtf:currentParaStyle] }
    if [info exists rtf_styleSheet($style,DEF)] {
	if {$rtf_styleSheet($style,TYPE) != "CHAR"} {
	    rtf:warning "$style not a character style"
	} else {
           rtf:write "\\cs$rtf_styleSheet($style,NUM)$rtf_styleSheet($style,DEF) "
	}
    } else {
	rtf:warning "No definition for character style <$style>"
    }
}

###
### Bookmarks
###

proc rtf:startBookmark { name } {
    rtf:bgroup				
    rtf:write "\\*\\bkmkstart $name"
    rtf:egroup
}

proc rtf:endBookmark { name } {
    rtf:bgroup
    rtf:write "\\*\\bkmkend $name"
    rtf:egroup
}

###
### Fields
###

proc rtf:startFieldInst {} {
    rtf:bgroup
        rtf:write "\\field\\flddirty"
        rtf:bgroup
            rtf:write "\\*\\fldinst "
}

proc rtf:startFieldResult {} {
        rtf:egroup
        rtf:bgroup
            rtf:write "\\fldrslt "
}

proc rtf:startField {inst} {
    rtf:startFieldInst
	    rtf:write [rtf:Escape $inst]
    rtf:startFieldResult
}

proc rtf:endField {} {
	rtf:egroup
    rtf:egroup
}

proc rtf:insertField {inst {rslt ""}} {
    rtf:startFieldInst
    #    rtf:write [rtf:Escape $inst]
    # Changed by BT to the following:
    rtf:write $inst
    rtf:startFieldResult
    rtf:write [rtf:Escape $rslt]
    rtf:endField
}
# %%% note: syntax productions in spec disagree with examples,
# %%% what Word actually does, and common sense.

###
### Destination groups
###
array set rtf_Destinations {
    Header		"header"
    LeftHeader		"headerl"
    RightHeader 	"headerr"
    FirstPageHeader	"headerf"
    Footer		"footer"
    FirstPageFooter	"footerf"
    LeftFooter  	"footerl"
    RightFooter  	"footerr"
    Footnote		"footnote"
    Endnote		"footnote\\ftnalt"
}
# %%% Should stack diversions, current state &c;
proc rtf:divert {diversion} {
    global rtf_state rtf_Destinations rtf_savedState
    if {$rtf_state(diversion) != ""} {
	rtf:warning "Diversion $diversion within $rtf_state(diversion)"
    }
    if ![info exists rtf_Destinations($diversion)] {
	rtf:warning "Bad destination $diversion"
	return
    }
    rtf:bgroup
    # StarOffice refuses to load these with leading \*. [BT]
    #rtf:write "\\*\\$rtf_Destinations($diversion)"

    rtf:write "\\$rtf_Destinations($diversion)"
    array set rtf_savedState [array get rtf_state]
    array set rtf_state {inpara 0 insection 0 incell 0 inrow 0 intable 0}
    set rtf_state(diversion) $diversion
    return
}

proc rtf:undivert {} {
    global rtf_state rtf_savedState
    if {$rtf_state(diversion) == ""} {
	rtf:warning "No current diversion"
	return
    }
    rtf:egroup
    array set rtf_state [array get rtf_savedState]
    unset rtf_savedState
    set rtf_state(diversion) ""
    return
}

proc rtf:CharSet {} {
    global TEItools_locale

    if { [info exist TEItools_locale] } {
	switch -regexp $TEItools_locale {
	    "(koi8-r)|(ru)" {
		return 204
	    }
	}
    }
    return 0
}
