#
# RTF Tables
#
# Test version $Revision: 1.4 $
#

array set rtf_state { intable 0  inrow 0  incell 0  cellno 0 }

# nb: CellPosition is really a dimension
array set rtf:TableAttrs {
    CellPosition	{integer "cellx"}
    RowGap		{dimension "trgaph"}
    RowHeight		{dimension "trrh"}
    HeadingRow		{flag "trhdr"}
    Align		{enum {
	{Left	"trql"}
	{Right	"trqr"}
	{Center	"trqc"}
    }}
    CellBorderBottom	{table rtf_RuleTable "clbrdrb"}
    CellBorderTop	{table rtf_RuleTable "clbrdrt"}
    CellBorderLeft	{table rtf_RuleTable "clbrdrl"}
    CellBorderRight	{table rtf_RuleTable "clbrdrr"}
    Direction		{enum {
	{BottomUp	"cltxlrtb"}
	{TopDown	"cltxtbrl"}
	{Normal		"clvertalt"}
    }}
     Merge		{enum {
 	{First	"clvmgf"}
 	{Merge  "clvmrg"}
     }}
}

global rtf_tableSettings rtf_tableDefaults
array set rtf_tableDefaults {
    width	6in
    numcols	1
    numcells	1
    rowgap	0pt
    rowsep	-
    colsep	-
    frame	-
    toprule	-
    botrule	-
    rowalign	Center
    headingrow	0
}
array set rtf_tableSettings [array get rtf_tableDefaults]

#
# Different ways to specify number and widths of columns:
#
#  + number of columns and total width (equal-sized columns)
#  + list of relative widths and total width
#  + list of absolute widths
# 
proc _rtf:tableRelativeWidths {colwidths} {
    global rtf_tableSettings rtf_cellPositions
    set tablewidth [rtf:cvTwips $rtf_tableSettings(width)]
    set sumwidths 0
    foreach colwidth $colwidths { incr sumwidths $colwidth }
    set colend 0
    set colnum 0
    foreach colwidth $colwidths {
	incr colend [expr round(($tablewidth*$colwidth)/$sumwidths)]
	incr colnum
	set rtf_cellPositions($colnum) $colend
    }
    set rtf_tableSettings(numcols) $colnum
}

proc _rtf:tableAbsoluteWidths {colwidths} {
    global rtf_tableSettings rtf_cellPositions
    set colend 0
    set colnum 0
    foreach colwidth $colwidths {
	incr colend [rtf:cvTwips $colwidth]
	incr colnum
	set rtf_cellPositions($colnum) $colend
    }
    set rtf_tableSettings(width) $colend
    set rtf_tableSettings(numcols) $colnum
}

proc _rtf:tableEqualWidths {numcols} {
    global rtf_tableSettings rtf_cellPositions
    set rtf_tableSettings(numcols) $numcols
    set tablewidth [rtf:cvTwips $rtf_tableSettings(width)]
    set colnum 0
    while {$colnum < $numcols} {
	incr colnum
	set rtf_cellPositions($colnum) \
		[expr round($tablewidth*$colnum/$numcols)]
    }
}

proc rtf:startTable {args} {
    global rtf_state rtf_tableSettings rtf_tableDefaults
    if $rtf_state(intable) {
	rtf:warning "Already in table"
	return;
    }
    array set rtf_tableSettings [array get rtf_tableDefaults]
    catch {unset rtf_cellPositions}
    set option ""
    foreach arg $args {
	if {$option == ""} { set option $arg; continue; }
	switch -- $option {
	    -width	{ set rtf_tableSettings(width) $arg }
	    -align	{ set rtf_tableSettings(rowalign) $arg }
	    -numcols 	{ _rtf:tableEqualWidths $arg }
	    -relwidths	{ _rtf:tableRelativeWidths $arg }
	    -abswidths	{ _rtf:tableAbsoluteWidths $arg }
	    -toprule 	{ set rtf_tableSettings(toprule) $arg }
	    -colsep 	{ set rtf_tableSettings(colsep) $arg }
	    -rowsep 	{ set rtf_tableSettings(rowsep) $arg }
	    -frame 	{ set rtf_tableSettings(frame) $arg
		set rtf_tableSettings(toprule) $arg }
	    -headingrow { set rtf_tableSettings(headingrow) $arg }
	    default { rtf:warning "rtf:startTable: bad option $option" }
	}
	set option ""
    }
    if {$option != ""} {
	rtf:warning "rtf:startTable: no argument for option $option"
    }
    set rtf_tableSettings(botrule) $rtf_tableSettings(rowsep)
    array set rtf_state { intable 1  inrow 0  incell 0  cellno 0 }
}

proc rtf:startRow {args} {
    global rtf_state rtf_tableSettings

    if !$rtf_state(intable) {
	rtf:warning "Not in a table"
    }
    if $rtf_state(inrow) rtf:endRow

    set isheading 0
    set colspans ""
    set rowspans ""
    set option ""
    foreach arg $args {
	if {$option == ""} { set option $arg; continue; }
	switch -- $option {
	    -colspans	{ set colspans $arg }
	    -rowspans   { set rowspans $arg }
	    -botrule 	{ set rtf_tableSettings(botrule) $arg }
	    -colsep 	{ set rtf_tableSettings(colsep) $arg }
	    -heading	{ set isheading $arg }
	    -coldirs	{ set coldirs $arg }
	    default	{
		rtf:warning "rtf:startRow: bad option $option" 
	    }
	}
	set option ""
    }
    if {$option != ""} {
	rtf:warning "rtf:startRow: no argument for option $option"
    }

    set rowatts [list \
		     RowGap	$rtf_tableSettings(rowgap) \
		     Align	$rtf_tableSettings(rowalign) \
		     HeadingRow $rtf_tableSettings(headingrow)]
    if $isheading {
	lappend rowatts HeadingRow $isheading
    }
    rtf:write "\\trowd[rtf:ExpandAttributes rtf:TableAttrs $rowatts]\n"

    if {[info exists coldirs] && $coldirs != ""} {
	set colno 1
	foreach direction $coldirs {
	    global rtf_cellDirections
	    set rtf_cellDirections($colno) $direction
	    incr colno
	}
    }
    
    if {$rowspans != ""} {
	set colno 1
	foreach merge $rowspans {
	    global rtf_cellMerges
	    if {$merge != "Normal"} {
		set rtf_cellMerges($colno) $merge
	    }
	    incr colno
	}
    }
    
    if {$colspans != ""} {
	set rtf_tableSettings(numcells) [llength $colspans]
	set colno 1
	foreach span $colspans {
	    rtf:cellDef $colno $span
	    incr colno $span
	}
    } else {
	set rtf_tableSettings(numcells) $rtf_tableSettings(numcols)
	for {set colno 1} {$colno <= $rtf_tableSettings(numcols)} {incr colno} {
	    rtf:cellDef $colno
	}
    }

    rtf:write "\n"
    array set rtf_state { inrow 1 incell 0 cellno 0 }
}

proc rtf:cellDef {colno {span 1}} {
    global rtf_tableSettings rtf_cellPositions
    set start $colno
    set end [expr $colno + $span - 1]
    if {$end > $rtf_tableSettings(numcols)} {
	rtf:warning "Cell spans too many columns"
	return
    }
    if {$start == 1} {
	set border(Left) $rtf_tableSettings(frame) 
    } else {
	set border(Left) $rtf_tableSettings(colsep) 
    }
    if {$end == $rtf_tableSettings(numcols)} {
	set border(Right) $rtf_tableSettings(frame) 
    } else { 
	set border(Right) $rtf_tableSettings(colsep)
    }
    set border(Top)   	$rtf_tableSettings(toprule)
    set border(Bottom)	$rtf_tableSettings(botrule)
    set cellAtts {}
    foreach b {Top Bottom Left Right} {
	if {$border($b) != "-"} {
	    lappend cellAtts "CellBorder$b" $border($b)
	}
    }

    global rtf_cellDirections
    if { [info exists rtf_cellDirections($colno)] } {
	lappend cellAtts Direction $rtf_cellDirections($colno)
    }

    global rtf_cellMerges
    if { [info exists rtf_cellMerges($colno)] } {
	lappend cellAtts Merge $rtf_cellMerges($colno)
    }
    
    # cellx must be last
    lappend cellAtts CellPosition $rtf_cellPositions($end)
    rtf:write [rtf:ExpandAttributes rtf:TableAttrs $cellAtts]
}

# ... need to distinguish between cells which contain paragraphs
# ... and those which only contain inline stuff;
# ... for the latter apply paragraph properties when the cell starts

proc rtf:startCell {{style -}} {
    global rtf_state
    if {$rtf_state(incell)} { rtf:endCell }
    set rtf_state(incell) 1
    incr rtf_state(cellno)
    rtf:startPara $style
}
proc rtf:endCell {} {
    global rtf_state
    if !$rtf_state(incell) { rtf:warning "Not in a cell"; return }
    rtf:write "\\cell\n"
    array set rtf_state { inpara 0 incell 0 }
}

proc rtf:endRow {} {
    global rtf_state rtf_tableSettings
    if !$rtf_state(inrow) {
	rtf:warning "Not in a table row"
	return
    }
    if $rtf_state(incell) rtf:endCell
    if {$rtf_state(cellno) != $rtf_tableSettings(numcells)} {
	rtf:warning \
	    "$rtf_state(cellno) cells; should be $rtf_tableSettings(numcells)"
    }
    rtf:write "\\row\n"
    set rtf_state(inrow) 0
    set rtf_tableSettings(toprule) $rtf_tableSettings(botrule)
    set rtf_tableSettings(botrule) $rtf_tableSettings(rowsep)
}

proc rtf:endTable {} {
    global rtf_state
    if !$rtf_state(intable) {
	rtf:warning "Not in a table"
	return
    }
    if $rtf_state(inrow) rtf:endRow
    # without this, Word sometimes crashes...
    rtf:write "\\pard\\par\n"
    set rtf_state(intable) 0
}


