# Common procedures for TEItools
# $Id: TEItools.tcl,v 1.62 1999/11/23 11:08:27 tobotras Exp $

proc typeVal {} {
    #
    return [attVal TYPE]
}

proc attVal { attr } {
    return [string tolower [query attval $attr]]
}

proc toUpper { text } {
    return [string toupper ${text}]
}

proc toLower { text } {
    return [string tolower ${text}]
}

proc currentDivNo {} {
    global splitNames splitLevel
    if { [info exists splitLevel] && $splitLevel > 0 } {
	set splitType [lindex $splitNames($splitLevel) 0]
    } else {
	set splitType "        "
    }
    set n {}
    foreachNode ancestor {
	if { [string match "DIV*" "[query gi]"] } {
	    if { "[query gi]" == "DIV1" || \
		     "[query gi]" == "DIV" && "[typeVal]" == $splitType } {
		if { "$n" != "" } {
		    set n ".${n}"
		}
		set divN [query propval divNumber]
		if { "$divN" == "" } {
		    return "[childNumber]${n}"
		}
		else {
		    return "${divN}${n}"
		}
	    }
	    if { "$n" != "" } {
		set n ".${n}"
	    }
	    set n "[childNumber]${n}"
	}
    }
    return $n
}

proc prepend { var val } {
    upvar 1 $var l
    set l "${val}${l}"
    return $l
}

proc pushMode { newSpec } {
    global translateSpec
    # We not always use Simple.tcl
    if { ![info exists translateSpec] } {
	set translateSpec NULL
    }
    if { [catch {pushMode_theMode get spec}] } {
	environment pushMode_theMode \
	    spec $translateSpec
    }
    pushMode_theMode save
    pushMode_theMode set spec $newSpec
    set translateSpec $newSpec
}

proc popMode {} {
    global translateSpec
    pushMode_theMode restore
    set translateSpec [pushMode_theMode get spec]
}

proc null { text } {
    return
}

#Not all of scripts use Simple.tcl, so define it here too:
proc identity { text } {
    return "${text}"
}

array set splitNames {
    0	"    division,    . @\#$"
    1	"section act"
    2	"subsection scene"
    3	"subsubsection"
    4   "paragraph"
    5	"subparagraph"
    6	"subsubparagraph"
    7	"subsubsubparagraph"
}

proc TEItools_load_style { styles } {
    foreach style [split $styles " ,"] {
	if { "$style" != "" } {
	    debug "Style to be loaded: $style"
	    TEItools_do_load $style
	}
    }
}

proc TEItools_do_load { style } {
    debug "do_load got `$style'"
    global TEItools_loaded_styles
    if { [info exists TEItools_loaded_styles($style)] } {
	Warning "Style $style is already loaded"
	return
    } else {
	set TEItools_loaded_styles($style) "Yes, I'm loaded already"
    }
    
    global TEItools_LOADING_STYLE_shown
    if { ![ info exists TEItools_LOADING_STYLE_shown] } {
	set TEItools_LOADING_STYLE_shown "Yes"
	puts -nonewline stderr "Loading styles: "
    }

    set eq [string first = $style]
    if { $eq == -1 } {
	puts -nonewline stderr "$style "
	TEItools_load_style_script $style 
    } else {
	set name [string range $style 0 [expr $eq - 1]]
	set value [string range $style [expr $eq + 1] end]
	global TEItools_${name}_value
	set TEItools_${name}_value $value
	puts -nonewline stderr "${name}=${value} "
    }
}

proc TEItools_load_style_script { script } {

    global TEItools_style_search_path
    
    if { ![info exists TEItools_style_search_path] } {
	set TEItools_style_search_path "."
    }
    
    set path [split $TEItools_style_search_path ":"]
    
    for {set idx 0} {$idx < [llength $path]} {incr idx} {
	set dir [lindex $path $idx]
	set file [file join $dir $script]
	if { [file readable $file] } {
	    global TEItools_styles_to_load
	    if { [info exists TEItools_styles_to_load] } {
		append TEItools_styles_to_load ":$file"
	    } else {
	        append TEItools_styles_to_load "$file"
	    }
	    global TEItools_${script}_in_use
	    set TEItools_${script}_in_use true
	    return
	}
    }
    puts stderr "
***ERROR: cannot find script file $script.
   $TEItools_style_search_path was searched.
"
}

proc TEItools_execute_style_scripts {} {
    debug "Loading styles..."
    global TEItools_styles_to_load
    if { [info exists TEItools_styles_to_load] } {
	set files [split $TEItools_styles_to_load ":"]
	for {set idx 0} {$idx < [llength $files]} {incr idx} {
	    debug "Loading [lindex $files $idx]..."
	    source [lindex $files $idx]
	}
	unset TEItools_styles_to_load
    }
    debug "Done."
}

proc TEItools_do_PI { text } {

    set s [split $text " \t"]
    set instruction [string tolower [lindex $s 0]]
    if { [llength $s] > 0 } {
	set args [lrange $s 1 [expr [llength $s] - 1]]
    } else {
	set args {}
    }

    set procName TEItools_PI_${instruction}

    if { [info proc $procName] == $procName } {
	eval $procName $args
    } else {
	puts stderr "\n***WARNING: Unknown processing instruction: $instruction"
    }
}

proc TEItools_PI_usestyle { name } {
    TEItools_load_style $name
    TEItools_execute_style_scripts
}

proc TEItools_PI_nextdivision { number } {
    withNode next element DIV1 {
	setprop divNumber $number
    }
}

proc repeat { string counter } {
    set ret {}
    while { $counter > 0 } {
	append ret $string
	incr counter -1
    }
    return $ret
}

proc openUnknown {} {
    Warning "unknown element [query gi]"
    return
}

proc stripFileType { filename } {
    set dotPosition [string last "." $filename]
    if { $dotPosition == -1 } {
	return $filename
    } else {
	return [string range $filename 0 [expr $dotPosition - 1]]
    }
}

proc isNumber { string } {
    if { [string length $string] == 0 } {
	return 0
    }
    set a [string index $string 0]
    if { [string compare $a "0"] >= 0 && [string compare $a "9"] <= 0 } {
	return 1
    } else {
	return 0
    }
}

proc TEItoolsSetup { newFileExt } {
    global outFile argv env TEItools_locale

    set TEItools_locale [query docroot child el attVal LANG]
    if { "$TEItools_locale" == "" } {
        if { [info exist env(SGML_LANG)] } {
            set TEItools_locale $env(SGML_LANG)
	}
    }
    if { "$TEItools_locale" == "" } {
        if { [info exist env(LANG)] } {
	    set TEItools_locale $env(LANG)
	}
    }
    if { "$TEItools_locale" == "" } {
        set TEItools_locale en
    } else {
    	set TEItools_locale [string tolower $TEItools_locale]
    }
    puts stderr "Loading $TEItools_locale locale ..."
    require locale.${TEItools_locale}.tcl
    debug "Locale done."
    if { "$newFileExt" != "" } {
	set outFile [open ${argv}.${newFileExt} w]
	global silence
	set silence 0
	proc output { text } {
	    global outFile silence
	    if { $silence == 0 } {
		puts $outFile $text nonewline
	    }
	}
	puts stderr "Transforming SGML ..."
    }
    debug "TEItoolsSetup done."
}

proc filteredContent {} {
    assert {"[info proc textSdataFilter]" == "textSdataFilter"}
    assert {"[info proc textCdataFilter]" == "textCdataFilter"}
    set text ""
    foreachNode child pel {
	append text [content]
    }
    return [textSdataFilter [textCdataFilter $text]]
}

substitution AddrToName {
    {:} {_}
}

proc firstChunk {} {
    withNode parent left {
	return 0
    }
    withNode left {
	return 0
    }
    return 1
}

proc lastChunk {} {
    withNode parent right {
	return 0
    }
    withNode right {
	return 0
    }
    return 1
}

proc inlineNoteFilter { text } {
    substitution noteSubst {
	"\n" " "
	"\t" " "
    }
    set text [noteSubst $text]
    if { [firstChunk] } {
	set text [trimLeadingSpace $text]
    }
    if { [lastChunk] } {
	set text [trimTrailingSpace $text]
    }
    return [textCdataFilter $text]
}

proc trimLeadingSpace { text } {
    set idx 0
    set len [string length $text]
    while { $idx < $len && [string index $text $idx] == " " } {
	incr idx
    }
    return [string range $text $idx end]
}

proc trimTrailingSpace { text } {
    set idx [expr [string length $text] - 1]
    while { $idx >= 0 && [string index $text $idx] == " " } {
	incr idx -1
    }
    return [string range $text 0 $idx]
}

proc trimSpaces { text } {
    return [string trim $text " \t\n"]
}

proc isdigit { c } {
    if { [string first $c "01234567890"] < 0 } {
	return 0
    } else {
	return 1
    }
}

proc debug { text } {
    global env
    if { [info exists env(TEITOOLS_DEBUG)] } {
	puts stderr "\n---DEBUG: \"$text\" in [ErrorTrace]"
    }
}

proc Debug { text } {
    debug $text
}

proc assert { condition } {
    if [subst $condition] return
    Error "Assertion failed: $condition"
    exit 1
}

proc Error { text } {
    puts stderr "\n***ERROR: $text in [ErrorTrace]"
}

proc Warning { text } {
    puts stderr "\n***WARNING: $text in [ErrorTrace]"
}

proc ErrorTrace {} {
    if { "[query nodetype]" == "SD" || "[query gi]" == "" } return
    set ret "([expr 1 + [countq prev element [query gi]]],[query gi])"
    withNode parent {
	set ret "[ErrorTrace]$ret"
    }
    return $ret
}

proc paragraphNumber {} {
    set ordered 0
    withNode ancestor withattval REND "ordered" {
	set ordered 1
    }

    if { $ordered == 0 } return

    set ret [childNumber]
    foreachNode ancestor {
	if { [string match "DIV*" [query gi]] } {
	    if { [attVal TYPE] == "appendix" } {
		set ret "[appendixLetter [appendixNumber]].$ret"
	    } else {
		set ret "[divisionNumber].$ret"
	    }
	}
    }
    return $ret
}

# childNumber, except for that <div type='stub'> are not counted
proc divisionNumber {} {
    set N 1
    foreachNode prev element [query gi] {
	if { "[attVal TYPE]" != "stub" } {
	    incr N
	}
    }
    return $N
}

proc appendixNumber {} {
    #Analog of childNumber, but counts to first <div type='appendix'>
    return [expr 1 + [countq prev el withGI [query gi] withattval TYPE APPENDIX]]
}

proc squeeze { string } {
    regsub -all "\[ \t\n\]+" $string " " s
    return [string trim $s]
}

proc eatNLs { text } {
    if { "$text" == "\n" } {
	return " "
    } else {
	return $text
    }
}

proc checkDivType {} {
    array set correctTypes {
	DIV0 (section)|(appendix)|(stub)|(act)
	DIV1 (section)|(appendix)|(stub)|(act)
	DIV2 (subsection)|(stub)|(scene)
	DIV3 (subsubsection)|(stub)
	DIV4 (paragraph)|(stub)
	DIV5 (subparagraph)|(stub)
	DIV6 (subsubparagraph)|(stub)
	DIV7 (subsubsubparagraph)|(stub)
	DIV8 (subsubsubsubparagraph)|(stub)
	DIV  (abstract)|(appendix)|(colophon)|(bibl)
    }
    set gi [query gi]
    if { ![info exists correctTypes($gi)] } return
    if { ![regexp $correctTypes($gi) [typeVal]] } {
	Error "<$gi> should have attribute TYPE set to `$correctTypes($gi)', not to `[typeVal]'"
    }
}

proc cutGraphicExtensionOff { filename } {
    switch -regexp [toLower [file extension $filename]] {
	"(^\.eps$)|(^\.gif$)|(^\.tiff$)|(^\.tif$)|(^\.jpeg$)|(^\.jpg$)|(^\.png$)" {
	    return [file rootname $filename]
	}
	default {
	    return $filename
	}
    }
}

proc XREF2HREF {} {
    set text "[query attval N]"
    if { $text == "" } {
       return [filteredContent]
    } else {
       return $text
    }
}
