############# TEI extended pointer #################################
# $Id: TEItoolsEPN.tcl,v 1.13 1999/09/15 09:00:51 tobotras Exp $

# NOTES
# ~~~~~
#
# 2. CHILD (ALL) is equal to CHILD (1).
# 3. Not implemented:
#     - regular;
#     - 'to' (spans);
#     - TOKEN;
#     - PATTERN;
#     - STR;
#     - SPACE;
#     - FOREIGN;
#     - HIQ.
# 4. Only one attribute-value pair from avspecs is used.

proc doXptr { actor } {

    debug "doXptr"
    
    set from [attVal FROM]
    set to [attVal TO]
    set doc [query attval DOC];		# do NOT ignore case here!

    return [XPointer $doc $from $to $actor]
}

proc XPointer { doc from to actor } {

    debug "XPointer { {$doc} {$from} {$to} }"
    
    if { "$from" == "" } {
	return [XPointer $doc "root" $to $actor]
    }

    if { "$to" == "" } {
	return [XPointer $doc $from "ditto" $actor]
    }
    
    if { "$doc" != "" } {
	set found 0
	withNode entity $doc {
	    #obtain filename
	    set doc [query sysid]
	    set found 1
	}
	if { $found == 0 } {
	    puts stderr "***ERROR: No such entity: $doc"
	    return
	}
	    
    }

    return [ProcessExtendedPointer $doc [squeeze $from] [squeeze $to] $actor]
}

proc ProcessExtendedPointer { fileName from to actor } {
    
    debug "ProcessExtendedPointer { {$fileName} {$from} {$to} }"

    global TEItools_EPNsysid
    set TEItools_EPNsysid $fileName
    
    if { "$fileName" != "" } {
	if { [catch  {PushDocument $fileName} err] } {
	    puts stderr "***ERROR: $err"
	    set needsPop 0
	} else {
	    set needsPop 1
	}
    } else {
	set needsPop 0
    }
    set ret [FetchEPN $from $to $actor]
    if { $needsPop } {
	PopDocument
    }
    return $ret
}

environment EPNdocumentStreams stream ""

proc PushDocument { fName } {

    set fName [string trim $fName]
    
    debug "PushDocument { $fName }"

    set str [savestream]

    debug "Saved current stream ($str)"
    
    EPNdocumentStreams set stream $str
    EPNdocumentStreams save
    
    global EPNloadedDocs
    if { [info exists EPNloadedDocs($fName)] } {
	set str $EPNloadedDocs($fName)
	debug "Already loaded, stream is $str."
	setstream $str
    } else {
	loadNewFile $fName
	set str [savestream]
	setstream $str
	set EPNloadedDocs($fName) $str
    }
}

proc PopDocument {} {

    debug "PopDocument"
    
    EPNdocumentStreams restore
    setstream [EPNdocumentStreams get stream]
}

proc loadNewFile { fName } {

    debug "loadNewFile { $fName }"
    
    if { ![ file readable $fName ] } {
	error "Can't read file $fName"
    }
    loaddoc $fName
}

# TEI EPN (near-)formal grammar

# ladder   ::=  locterm
#          |    ladder locterm

# locterm ::=   'ROOT'                        // default first location
#          |    'HERE'                        // location of the xptr
#          |    'ID' '(' NAME ')'             // only one ID allowed.
#          |    'REF' '(' characters ')'      // only one ref allowed
#          |    'CHILD' steps
#          |    'DESCENDANT' steps
#          |    'ANCESTOR' steps
#          |    'PREVIOUS' steps
#          |    'NEXT' steps
#          |    'PRECEDING' steps
#          |    'FOLLOWING' steps
#          |    'PATTERN' regs                // mult patterns allowed
#          |    'TOKEN' '(' range ')'
#          |    'STR' '(' range ')'
#          |    'SPACE' '(' NAME ')' pointpair
#          |    'FOREIGN' parms
#          |    'HYQ' parms
#          |    'DITTO'                       // valid only in TO att.

# steps    ::=  '(' step ')'
#          |    steps '(' step ')'
 
# step     ::=  instance
#          |    instance element
#          |    instance element avspecs
 
# avspecs  ::=  attribute value
#          |    avspecs attribute value

# instance ::=  'ALL'
#          |    signed
 
# signed   ::=  NUMBER                        // default sign is +
#          |    '+' NUMBER
#          |    '-' NUMBER
      
# element  ::=  NAME
#          |    '#CDATA'
#          |    '*'
#          |    '(' regular ')'

# attribute ::= NAME
#          |    '*'
#          |    '(' regular ')'    

# value    ::=  LITERAL                  // i.e. quoted string.
#          |    NAME                     // As for attribute values in
#          |    NUMBER                   // document, NMTOKENs need not
#          |    NUMTOKEN                 // be quoted
#          |    '#IMPLIED'               // No value specified, no default
#          |    '*'                      // Any value matches.
#          |    '(' regular ')'

# regs     ::= '(' regular ')'
#          |    regs '(' regular ')'
 
# regular  ::=  character
#          |    '.'                           // match any character
#          |    '[&circ' || characters || ']'     // match any char not in list
#          |    '[' || characters || ']'      // match any char in list
#          |    '\a'                     // match any alphabetic
#          |    '\d'                     // match any digit 0-9
#          |    '\n'                     // match newline (&#RE;&#RS;)
#          |    '\s'                     // match any whitespace character
#          |    '\\'                     // match backslash (rev. solidus)
#          |    '\' || nonspecial        // match nonspecial character
#          |    regular || '*'           // match 0-n of 'regular'
#          |    regular || '+'           // match 1-n of 'regular'
#          |    regular || '?'           // match 0-1 of 'regular'
#          |    '&circ' || regular           // match at start of loc source
#          |    regular || '$'           // match at end of loc source
#          |    regular || regular       // match 1st, then 2d regular exp.
#          |    regular || '|' || regular // match either 1st or 2d
#          |    '(' || regular || ')'    // use parentheses for grouping
 
# characters ::= /* empty string */
#          |    characters character
 
# nonspecial ::= /* any character except a, d, n, or s */

# range    ::=  NUMBER
#          |    NUMBER NUMBER							 


# pointpair ::= '(' numbers ')'
#          |    '(' numbers ')' '(' numbers ')'
 
# numbers  ::=  signed
#          |    numbers signed


proc FetchEPN { from to actor } {
    debug "FetchEPN { $from }"

    if { $to != "ditto" } {
	error "spans in EPN are not implemented yet, sorry"
    }

    global EPNabsolute
    set EPNabsolute 0
    
    while { "$from" != "" } {
	debug "->> ladder \"$from\""
	XPointerStep from
    }
    
    return [$actor]
}

array set EPNtoCOST {
    child	child
    descendant	descendant
    ancestor	ancestor
    previous	prev
    next	next
    preceding	backward
    following	forward
}

proc XPointerStep { lName } {

    upvar $lName ladder

    debug "XPointerStep {$ladder}"
    set savedLadder $ladder
    
    set keyWord [eatWord ladder]

    debug " kw '$keyWord', rest of ladder '$ladder'"

    global EPNabsolute
    if { $keyWord != "root" && $keyWord != "here" && $EPNabsolute == 0 } {
	selectNode docroot child el
	set EPNabsolute 1
    }
    
    switch -regexp $keyWord {
	"root" {
	    selectNode docroot child el
	}
	"here" {
	    # do nothing
	}
	"(id)|(ref)" {
	    debug ">>id/ref"
	    set expression [eatExpression $lName]
	    set address ""
	    debug ">> expr '$expression'"
	    if { $keyWord == "id" } {
		set attr "ID"
	    } else {
		set attr "N"
	    }
	    withNode descendant withattval $attr $expression {
		    set address [query address]
	    }
	    if { "$address" == "" } {
		Error "There is no node with $attr \"$expression\""
	    } else {
		selectNode node $address
		debug "found node, <[query gi] $attr='[attVal ID]'>"
	    }
	}
	"(child)|(descendant)|(ancestor)|(previous)|(next)|(preceding)|(following)" {
	    debug "steps-involing keyword, ladder '$ladder'"
	    while { [string index $ladder 0] == "(" } {
		# dirty trick here. Assuming CoST keywords are equal in syntax
		# and semantics with EPN keywords.
		global EPNtoCOST
		set keyWord $EPNtoCOST($keyWord)
		set step [eatExpression $lName]
		array set s [parseStep $step]
		#let's count candidates to resolve negative instance no
		set total 0
		if { "$s(element)" != "" } {
		    set elementExpr "element $s(element)"
		} else {
		    set elementExpr "el"
		}
		set attributeExpr ""
		if { [info exists s(attributes)] } {
		    array set attrs "$s(attributes)"
		    foreach attribute "[array names attrs]" {
			set attributeExpr "withattrval $attribute $attrs($attribute)"
			break
		    }
		}
		    
		debug "we're in [query gi]@[query address], doing 'foreachNode $keyWord $elementExpr $attributeExpr'"
		#we all love Tcl syntax, right?
		eval foreachNode $keyWord $elementExpr $attributeExpr {{
		    incr total
		}}
		debug "total nodes to choose from: $total"
		if { $s(instance) < 0 } {
		    set s(instance) [expr $s(instance) + $total]
		} else {
		    incr s(instance) -1
		}
		set idx 0
		set address ""
		debug "counting to $s(instance) instance"
		eval foreachNode $keyWord $elementExpr {{
		    if { $idx == $s(instance) } {
			set address [query address]
			break
		    }
		    incr idx
		}}
		if { "$address" == "" } {
		    Error "There is no node found for $savedLadder"
		    return
		}
		selectNode node $address
		debug "found node"
	    }
	}
	default {
	    Error "EPN syntax is not implemented yet, sorry: \"$keyWord\""
	    return
	}
    }
}

proc eatExpression { lName } {
    upvar 2 $lName ladder

    debug "eatExpression {$ladder}"
    
    if { [string index $ladder 0] == "(" } {
	set len [string length $ladder]
	set depth 0
	for { set idx 1} {$idx < $len} {incr idx} {
	    switch [string index $ladder $idx] {
		")" {
		    if { $depth > 0 } {
			incr depth -1
		    } else {
			set expression [string trim [string range $ladder 0 $idx] "()"]
			incr idx
			set ladder [string trimleft [string range $ladder $idx end]]
			return $expression
		    }
		}
		"(" {
		    incr depth
		}
	    }
	}
	error "Syntax error in EPN: non-balanced parenthesis in \"$ladder\""
    } else {
	return [eatWord ladder]
    }
}

# step     ::=  instance
#          |    instance element
#          |    instance element avspecs

proc parseStep {step} {

    debug "parseStep {$step}"

    #let's ignore avspecs for now

    set a(instance) [eatWord step]
    if { $a(instance) == "all" } {
	set a(instance) 1
    }
    set a(element) [eatWord step]
    if { "$a(element)" == "*" } {
	set a(element) ""
    }
    while { $step != "" } {
	set attribute [eatWord step]
	set value [eatWord step]
	set attr($attribute) $value
	set a(attributes) [array get attr]
    }
    return "[array get a]"
}

proc eatWord { varName } {
    upvar $varName line

    debug "eatWord {$line}"
    
    if { "$line" == "" } {
	return ""
    }
    if { [string index $line 0] == "\"" } {
	set len [string length $line]
	for {set idx 1} {$idx < $len} {incr idx} {
	    if { [string index $line $idx] == "\"" } {
		set wordEnd [expr $idx + 1]
		set word [string range $line 1 [expr $idx - 1]]
		break
	    }
	    if { $idx == $len } {
		error "Unclosed quote in `$line'"
	    }
	}
    } else {
	set wordEnd [string wordend $line 0]
	set word [string range $line 0 $wordEnd]
    }
    set line [string trimleft [string range $line [incr wordEnd] end]]
    return [string trimright $word]
}

proc titlePartSuffix {} {
    withNode next element TITLEPART {
	return " "
    }
    return
}

specification extractDocTitle_tex {
    {element TITLEPART} {
	suffix		[titlePartSuffix]
	cdataFilter	identity
	sdataFilter	textSdataFilter
    }
    {element Q} {
	prefix		"[openQuote]"
	suffix		"[closeRend]"
	cdataFilter	identity
	sdataFilter	textSdataFilter
    }
    {el} {
	startAction {
	    debug "found [query gi]"
	}
	cdataFilter	null
	sdataFilter	null
    }
}

specification extractHead_tex {
    {element Q} {
	prefix	"[openQuote]"
	suffix	"[closeRend]"
    }
    {el} {
	startAction {
	    debug "found [query gi]"
	}
	cdataFilter	textCdataFilter
	sdataFilter	textSdataFilter
    }
}

specification extractDocTitle_rtf {
    {element TITLEPART} {
	suffix		[titlePartSuffix]
	cdataFilter	identity
	sdataFilter	textSdataFilter
    }
    {element Q} {
	prefix		[oqIfNotBlock]
	suffix		[cqIfNotBlock]
	cdataFilter	identity
	sdataFilter	textSdataFilter
    }
    {el} {
	startAction {
	    debug "found [query gi]"
	}
	cdataFilter	null
	sdataFilter	null
    }
}

specification extractHead_rtf {
    {element Q} {
	prefix	"[openQuote]"
	suffix	"[closeRend]"
    }
    {el} {
	startAction {
	    debug "found [query gi]"
	}
	cdataFilter	identity
	sdataFilter	textSdataFilter
    }
}

proc sysIdToBasename { id } {
    return [file rootname $id].html
}

specification extractDocTitle_html {
    {element TITLEPAGE} {
	prefix		"<[Link] HREF='[sysIdToBasename $TEItools_EPNsysid]'>"
	suffix		"</A>"
    }
    {element TITLEPART} {
	suffix		[titlePartSuffix]
	cdataFilter	identity
	sdataFilter	textSdataFilter
    }
    {element Q} {
	prefix		"[openQuote]"
	suffix		"[closeRend]"
	cdataFilter	identity
	sdataFilter	textSdataFilter
    }
    {el} {
	startAction {
	    debug "found [query gi]"
	}
	cdataFilter	null
	sdataFilter	null
    }
}

specification extractHead_html {
    {element Q} {
	prefix	"[openQuote]"
	suffix	"[closeRend]"
    }
    {el} {
	startAction {
	    debug "found [query gi]"
	}
	cdataFilter	identity
	sdataFilter	textSdataFilter
    }
}

proc NodeReferenceName {} {

    debug "NodeReferenceName, we're in [query gi] at [query address]"

    set currentNode [query address]
    
    withNode docroot child el {
	if { [query address] == $currentNode } {
	    return [DocumentReferenceName]
	}
    }
    
    withNode child element HEAD {
	global TEItools_backend
	if { [info commands extractHead_${TEItools_backend}] == "" } {
	    error "*Internal*: extractHead_${TEItools_backend} not defined"
	}
	pushMode extractHead_${TEItools_backend}
	if { "[info proc output]" != "" } {
	    rename output NodeReferenceName_output
	}
	global EPNdocHead
	set EPNdocHead ""
	proc output { text } {
	    global EPNdocHead
	    append EPNdocHead $text
	}
	translateHandler
	rename output {}
	if { "[info proc NodeReferenceName_output]" != "" } {
	    rename NodeReferenceName_output output
	}
	popMode
	return $EPNdocHead
    }
    debug "No head in node"
    error "Can't find <HEAD> in [query gi], don't know how to refer to it."
}

proc xptrSyntaxError { ep } {
    puts stderr "***ERROR: Extended pointer syntax error in \"$ep\""
    return "\[XPTR syntax error\]"
}

proc DocumentReferenceName {} {

    debug "DocumentReferenceName"
    
    global TEItools_backend
    if { [info commands extractDocTitle_${TEItools_backend}] == "" } {
	error "*Internal*: extractDocTitle_${TEItools_backend} not defined"
    }
    pushMode extractDocTitle_${TEItools_backend}
    # ----------
    if { "[info proc output]" != "" } {
	rename output DocumentReferenceName_output
    }
    global EPNdocTitle
    set EPNdocTitle ""
    proc output { text } {
	global EPNdocTitle
	append EPNdocTitle $text
    }
    withNode docroot descendant element TITLEPAGE {
	translateHandler
    }
    rename output {}
    if { "[info proc DocumentReferenceName_output]" != "" } {
	rename DocumentReferenceName_output output
    }
    # ----------
    popMode
    return $EPNdocTitle
}

