# Common functions for TeX backends
# $Id: TEItoolsTeX.tcl,v 1.46 1999/11/23 12:16:40 tobotras Exp $

if { [catch {localize next} err] } {
    Error "*INTERNAL ERROR*: TEItoolsTeX included before TEItoolsSetup was called!"
    exit 1
}

# - FILTERS -

# Yes, this IS ugly. No, you are not supposed to understand that. But!
# Trust me, this is the only way to achieve the goal.

proc textSdataFilter { text } {
    set text [textFilterSdata $text]
    regsub -all -- {\[mdash \]} $text "[localize texMdash]" text
    regsub -all -- {\[laquo \]} $text "[localize lq]" text
    regsub -all -- {\[raquo \]} $text "[localize rq]" text
    return $text
}

proc verbatimFilter { text } {
    set text [verbatimFilterSubst $text]
    regsub -all -- {\[mdash \]} $text "[localize texMdash]" text1
    regsub -all -- {\[laquo \]} $text1 "[localize lq]" text
    regsub -all -- {\[raquo \]} $text "[localize rq]" text1
    return $text1
}

proc typewriterCdataFilter { text } {
    regsub -all "\[ \t\n\]+" [typewriterFilterCdata $text] "\n" text
    return $text
}

proc typewriterSdataFilter { text } {
    set text [typewriterFilterSdata $text]
    regsub -all {\[mdash \]} $text "[localize texMdash]" text
    regsub -all -- {\[laquo \]} $text "\"" text
    regsub -all -- {\[raquo \]} $text "\"" text
    return $text
}

#No Tcl-unsafe expressions in right side! (Like []s etc). All such
# cases should be replaced in textSdataFilter above!!

substitution textFilterSdata {
    {[AElig ]} {\AE }
    {[agrave]} {\`a}
    {[aacute]} {\'a}
    {[Aacute]} {\'A}
    {[acirc ]} {\^a}
    {[Acirc ]} {\^A}
    {[Agrave]} {\`A}
    {[aring ]} {\r{a}}
    {[Aring ]} {\r{A}}
    {[atilde]} {\~a}
    {[Atilde]} {\~A}
    {[auml  ]} {\"a}
    {[Auml  ]} {\"A}
    {[aelig ]} {\ae }
    {[ccedil]} {\c{c}}
    {[Ccedil]} {\c{C}}
    {[eacute]} {\'e}
    {[Eacute]} {\'E}
    {[ecirc ]} {\^e}
    {[Ecirc ]} {\^E}
    {[egrave]} {\`e}
    {[Egrave]} {\`E}
    {[euml  ]} {\"e}
    {[Euml  ]} {\"E}
    {[iacute]} {\'\i}
    {[Iacute]} {\'I}
    {[icirc ]} {\^\i}
    {[Icirc ]} {\^I}
    {[igrave]} {\`\i}
    {[Igrave]} {\`I}
    {[iuml  ]} {\"\i}
    {[Iuml  ]} {\"I}
    {[ntilde]} {\~n}
    {[Ntilde]} {\~N}
    {[oacute]} {\'o}
    {[Oacute]} {\'O}
    {[ocirc ]} {\^o}
    {[Ocirc ]} {\^O}
    {[ograve]} {\`o}
    {[Ograve]} {\`O}
    {[oslash]} {\o}
    {[Oslash]} {\O}
    {[otilde]} {\~o}
    {[Otilde]} {\~O}
    {[ouml  ]} {\"o}
    {[Ouml  ]} {\"O}
    {[szlig ]} {\ss }
    {[uacute]} {\'u}
    {[Uacute]} {\'U}
    {[ucirc ]} {\^u}
    {[Ucirc ]} {\^U}
    {[ugrave]} {\`u}
    {[Ugrave]} {\`U}
    {[uuml  ]} {\"u}
    {[Uuml  ]} {\"U}
    {[yacute]} {\'y}
    {[Yacute]} {\'Y}
    {[yuml  ]} {\"y}
    {[thorn ]} {\thorn }
    {[THORN ]} {\THORN }
    {[eth   ]} {\eth }
    {[ETH   ]} {\ETH }
    {[quot  ]} {\verb+"+}
    {[ldquo ]} {``}
    {[rdquo ]} {''}
    {[ndash ]} {{--}}
    {[numero]} {\textnumero{}}
    {[deg   ]} {\TEItoolsTextDegree{}}
    {[percnt]} {\%}
    {[hellip]} {\dots{}}
    {[rsquo ]} {'}
    {[lsquo ]} {`}
    {[excl  ]} {!}
    {[num   ]} {\#}
    {[dollar]} {\$}
    {[lsqb  ]} {[}
    {[rsqb  ]} {]}
    {[circ  ]} {\^}
    {[bsol  ]} {\textbackslash{}}
    {[grave ]} {`}
    {[lcub  ]} {\{}
    {[rcub  ]} {\}}
    {[copy  ]} {\textcopyright{}}
    {[tilde ]} {\textasciitilde{}}
    {[verbar]} {\textbar{}}
    {[lt    ]} {\textless{}}
    {[gt    ]} {\textgreater{}}
    {[plusmn]} {\ensuremath{\pm}}
    {[reg   ]} {\textregistered{}}
    {[check ]} {\checkmark{}}
    {[amp   ]} {\&}
    {[nbsp  ]} {~}
    {[hyphen]} {\mbox{-}}
    {[sharp ]} {\#}
    {[lowbar]} {\_}
    {[latex ]} {\LaTeX{}}
    {[tex   ]} {\TeX{}}
}

proc textCdataFilter { text } {
    return [textCdataFilterSubst $text]
}

substitution textCdataFilterSubst {
    {%} {\%}
    {&} {\&}
    {$} {\$}
    {-} {{--}}
    "#" {\#}
    {^} {\textasciicircum{}}
    {_} {\_}
    {~} {\textasciitilde{}}
    "\\" {\textbackslash{}}
    {<} {\textless{}}
    {>} {\textgreater{}}
	{|} {\textbar{}}
    "{" "\\{"
    "}" "\\}"
}
    
substitution verbatimFilterSubst {
    {[AElig ]} {\AE }
    {[agrave]} {\`a}
    {[aacute]} {\'a}
    {[Aacute]} {\'A}
    {[acirc ]} {\^a}
    {[Acirc ]} {\^A}
    {[Agrave]} {\`A}
    {[aring ]} {\r{a}}
    {[Aring ]} {\r{A}}
    {[atilde]} {\~a}
    {[Atilde]} {\~A}
    {[auml  ]} {\"a}
    {[Auml  ]} {\"A}
    {[aelig ]} {\ae }
    {[ccedil]} {\c{c}}
    {[Ccedil]} {\c{C}}
    {[eacute]} {\'e}
    {[Eacute]} {\'E}
    {[ecirc ]} {\^e}
    {[Ecirc ]} {\^E}
    {[egrave]} {\`e}
    {[Egrave]} {\`E}
    {[euml  ]} {\"e}
    {[Euml  ]} {\"E}
    {[iacute]} {\'\i}
    {[Iacute]} {\'I}
    {[icirc ]} {\^\i}
    {[Icirc ]} {\^I}
    {[igrave]} {\`\i}
    {[Igrave]} {\`I}
    {[iuml  ]} {\"\i}
    {[Iuml  ]} {\"I}
    {[ntilde]} {\~n}
    {[Ntilde]} {\~N}
    {[oacute]} {\'o}
    {[Oacute]} {\'O}
    {[ocirc ]} {\^o}
    {[Ocirc ]} {\^O}
    {[ograve]} {\`o}
    {[Ograve]} {\`O}
    {[oslash]} {\o}
    {[Oslash]} {\O}
    {[otilde]} {\~o}
    {[Otilde]} {\~O}
    {[ouml  ]} {\"o}
    {[Ouml  ]} {\"O}
    {[szlig ]} {\ss }
    {[uacute]} {\'u}
    {[Uacute]} {\'U}
    {[ucirc ]} {\^u}
    {[Ucirc ]} {\^U}
    {[ugrave]} {\`u}
    {[Ugrave]} {\`U}
    {[uuml  ]} {\"u}
    {[Uuml  ]} {\"U}
    {[yacute]} {\'y}
    {[Yacute]} {\'Y}
    {[yuml  ]} {\"y}
    {[thorn ]} {\thorn }
    {[THORN ]} {\THORN }
    {[eth   ]} {\eth }
    {[ETH   ]} {\ETH }
    {[quot  ]} {\verb+"+}
    {[ldquo ]} {``}
    {[rdquo ]} {''}
    {[ndash ]} {-}
    {[numero]} {\textnumero{}}
    {[deg   ]} {[puts stderr "***ERROR: &deg; can't be used in verbatim environment!"]}
    {[percnt]} {%}
    {[hellip]} {\dots }
    {[rsquo ]} {'}
    {[lsquo ]} {`}
    {[excl  ]} {!}
    {[num   ]} {\#}
    {[dollar]} {\$}
    {[lsqb  ]} {[}
    {[rsqb  ]} {]}
    {[circ  ]} {\^}
    {[bsol  ]} {\textbackslash{}}
    {[grave ]} {`}
    {[lcub  ]} {\{}
    {[rcub  ]} {\}}
    {[tilde ]} \~{}
    {[verbar]} {|}
    {[lowbar]} {_}
    {[lt    ]} {<}
    {[gt    ]} {>}
    {[amp   ]} {&}
    {[hyphen]} {\mbox{-}}
}

substitution typewriterFilterSdata {
    {[AElig ]} {\AE }
    {[agrave]} {\`a}
    {[aacute]} {\'a}
    {[Aacute]} {\'A}
    {[acirc ]} {\^a}
    {[Acirc ]} {\^A}
    {[Agrave]} {\`A}
    {[aring ]} {\r{a}}
    {[Aring ]} {\r{A}}
    {[atilde]} {\~a}
    {[Atilde]} {\~A}
    {[auml  ]} {\"a}
    {[Auml  ]} {\"A}
    {[aelig ]} {\ae }
    {[ccedil]} {\c{c}}
    {[Ccedil]} {\c{C}}
    {[eacute]} {\'e}
    {[Eacute]} {\'E}
    {[ecirc ]} {\^e}
    {[Ecirc ]} {\^E}
    {[egrave]} {\`e}
    {[Egrave]} {\`E}
    {[euml  ]} {\"e}
    {[Euml  ]} {\"E}
    {[iacute]} {\'\i}
    {[Iacute]} {\'I}
    {[icirc ]} {\^\i}
    {[Icirc ]} {\^I}
    {[igrave]} {\`\i}
    {[Igrave]} {\`I}
    {[iuml  ]} {\"\i}
    {[Iuml  ]} {\"I}
    {[ntilde]} {\~n}
    {[Ntilde]} {\~N}
    {[oacute]} {\'o}
    {[Oacute]} {\'O}
    {[ocirc ]} {\^o}
    {[Ocirc ]} {\^O}
    {[ograve]} {\`o}
    {[Ograve]} {\`O}
    {[oslash]} {\o}
    {[Oslash]} {\O}
    {[otilde]} {\~o}
    {[Otilde]} {\~O}
    {[ouml  ]} {\"o}
    {[Ouml  ]} {\"O}
    {[szlig ]} {\ss }
    {[uacute]} {\'u}
    {[Uacute]} {\'U}
    {[ucirc ]} {\^u}
    {[Ucirc ]} {\^U}
    {[ugrave]} {\`u}
    {[Ugrave]} {\`U}
    {[uuml  ]} {\"u}
    {[Uuml  ]} {\"U}
    {[yacute]} {\'y}
    {[Yacute]} {\'Y}
    {[yuml  ]} {\"y}
    {[thorn ]} {\thorn }
    {[THORN ]} {\THORN }
    {[eth   ]} {\eth }
    {[ETH   ]} {\ETH }
    {[quot  ]} {\verb+"+}
    {[ldquo ]} {``}
    {[rdquo ]} {''}
    {[ndash ]} {-}
    {[numero]} {\textnumero{}}
    {[deg   ]} {[puts stderr "***ERROR: &deg; can't be used in verbatim environment!"]}
    {[percnt]} {\%}
    {[hellip]} {...}
    {[rsquo ]} {'}
    {[lsquo ]} {`}
    {[excl  ]} {!}
    {[num   ]} {\#}
    {[dollar]} {\$}
    {[lsqb  ]} {[}
    {[rsqb  ]} {]}
    {[circ  ]} {\^}
    {[bsol  ]} {\textbackslash{}}
    {[grave ]} {`}
    {[lcub  ]} {\{}
    {[rcub  ]} {\}}
    {[tilde ]} {\textasciitilde{}}
    {[verbar]} {\textbar{}}
    {[lowbar]} {\_}
    {[lt    ]} {\textless{}}
    {[gt    ]} {\textgreater{}}
    {[amp   ]} {\&}
    {[nbsp  ]} {~}
    {[hyphen]} {\mbox{-}}
}

substitution typewriterFilterCdata {
    {%} {\%}
    {&} {\&}
    {$} {\$}
    {-} {-}
    "#" {\#}
    {^} {\textasciicircum{}}
    {_} {\_}
    {~} {\textasciitilde{}}
    "\\" {\textbackslash{}}
    {<} {\textless{}}
    {>} {\textgreater{}}
    {|} {\textbar{}}
    "{" "\\{"
    "}" "\\}"
}

# Do NOT EVER redefine this! This will eventually go into teitools.sty package.
proc TEItools_Style {} {
    return "
\\sloppy\\makeindex
\\setcounter{totalnumber}{5}

\\renewcommand{\\floatpagefraction}{.2}
\\renewcommand{\\dblfloatpagefraction}{\\floatpagefraction}
\\newcommand{\\TEItoolsTextDegree}{\\ensuremath{^\\circ}}
\\makeatletter
\\ifx\\undefined\\textnumero\\newcommand{\\textnumero}{\\TEItoolsTextNumero}\\fi
\\ifx\\undefined\\frontmatter\\newcommand{\\frontmatter}{\\pagenumbering{roman}}\\fi
\\ifx\\undefined\\mainmatter\\newcommand{\\mainmatter}{\\pagenumbering{arabic}}\\fi
\\ifx\\undefined\\backmatter\\newcommand{\\backmatter}{}\\fi
\\newcommand{\\TEItoolsTextNumero}{\\ifx\\undefined\\No N\\else[localize cyr]\\No\\fi{}}
\\newlength{\\TEItoolsFloatLen}
\\newbox{\\TEItoolsFloatFigBox}
\\newcommand{\\TEItoolsTableHead}{}

\\def\\@listi{\\topsep 0.4ex \\parsep 0pt \\itemsep \\parsep}
\\def\\@listi{\\topsep 0.4ex \\parsep 0pt \\itemsep \\parsep}
\\let\\@listI\\@listi
\\topsep 0.4ex \\partopsep 0pt \\itemsep 0pt \\parsep 0pt
\\def\\@listii{\\leftmargin\\leftmarginii
 \\labelwidth\\leftmarginii\\advance\\labelwidth-\\labelsep
 \\topsep 0pt \\parsep 0pt \\itemsep \\parsep}
\\def\\@listiii{\\leftmargin\\leftmarginiii
 \\labelwidth\\leftmarginiii\\advance\\labelwidth-\\labelsep
 \\parsep \\z@ \\topsep 0pt \\partopsep 0pt \\itemsep \\topsep}
\\makeatother 
\\addtolength{\\intextsep}{2\\baselineskip}
"
}

proc hyperrefArgs {} {
    return
}

proc extraPreamble {} {
    set ret ""
    global TEItools_skip_value
    if { [info exists TEItools_skip_value] } {
	append ret "\\renewcommand{\\baselinestretch}{$TEItools_skip_value}\n"
    }
    global TEItools_numdepth_value
    if { [info exists TEItools_numdepth_value] } {
	append ret "\\setcounter{secnumdepth}{$TEItools_numdepth_value}\n"
    }
    return "${ret}
"
}

proc sectionTitles {} {
	return "
%\\usepackage{titlesec}
%\\titlelabel{\\thetitle.\\quad}
%\\titleformat{\\paragraph}\[hang\]{\\bfseries}{\\theparagraph.}{.5em}{}

% Deep sections
\\makeatletter
\\newcounter{subsubparagraph}\[subparagraph\]
\\newcounter{subsubsubparagraph}\[subsubparagraph\]
\\newcommand\\subsubparagraph{\\@startsection{subsubparagraph}{6}{\\parindent}%
                                       {1.25ex \\@plus1ex \\@minus .2ex}%
                                       {1em}%
                                      {\\normalfont\\normalsize\\bfseries}}
\\newcommand\\subsubsubparagraph{\\@startsection{subsubsubparagraph}{7}{\\parindent}%
                                       {1.25ex \\@plus1ex \\@minus .2ex}%
                                       {1em}%
                                      {\\normalfont\\normalsize\\slshape}}
\\newcommand*\\l@subsubparagraph{\\@dottedtocline{6}{10em}{5em}}
\\newcommand*\\l@subsubsubparagraph{\\@dottedtocline{7}{10em}{5em}}
\\renewcommand\\thesubsubparagraph{\\thesubparagraph.\\@arabic\\c@subsubparagraph}
\\renewcommand\\thesubsubsubparagraph{\\thesubsubparagraph.\\@arabic\\c@subsubsubparagraph}
\\let\\subsubparagraphmark\\@gobble
\\let\\subsubsubparagraphmark\\@gobble

%\\newcommand{\\subsubparagraph}\[1\]{\\textsl\\textbf{{#1}}\\mbox{}\\par\\par}
%\\newcommand{\\subsubsubparagraph}\[1\]{\\textsl{#1}\\mbox{}\\par\\par}
\\makeatother
%-end of deep sections


% Section titles
%\\def\\section{\\@startsection{section}{1}
%{\\z@}{-2.5ex plus -0.5ex minus -0.1ex}{0.5ex plus 0.1ex}{\\raggedright\\Large\\bf}}
%\\def\\subsection{\\@startsection{subsection}{2}
%{\\z@}{-2.25ex plus -0.3ex minus -0.2ex}{0.05ex plus 0.05ex}{\\raggedright\\large\\bf}}
%\\def\\subsubsection{\\@startsection{subsubsection}{3}
%{\\z@}{-2.25ex plus -0.3ex minus -0.2ex}{0.05ex plus 0.05ex}{\\raggedright\\normalsize\\bf}}
%\\def\\paragraph{\\@startsection{paragraph}{4}
%{\\z@}{2ex plus 0.5ex minus 0.1ex}{0.05ex plus 0.05ex}{\\normalsize\\sf}}
%\\def\\subparagraph{\\@startsection{subparagraph}{5}
%{\\parindent}{2ex plus 0.5ex minus 0.1ex}{0.05ex plus 0.05ex}{\\normalsize\\it}}
%\\def\\@makechapterhead#1{\\vspace*{20pt}%
%{\\parindent 0pt\\Large\\bf \\ifnum\\c@secnumdepth>\\m@ne\\thechapter \\hskip 1em \\fi
% \\parbox\[t\]{0.7\\textwidth}{\\raggedright #1}\\par\\nobreak \\vskip25pt} }
%\\def\\@makeschapterhead#1{
% \\vspace*{20pt} { \\parindent 0pt \\raggedright
% \\LARGE\\bf#1\\par\\nobreak\\vskip25pt } }
"    
}

proc pageLayout {} {
    return
}

proc documentClassOptions {} {
    return "titlepage"
}

proc latexFileFooter {} {
    set fqdn {}
    global env
    if { [info exist env(HOST) ] } {
	set fqdn $env(HOST)
    } elseif { [info exist env(HOSTNAME)] } {
	set fqdn $env(HOSTNAME)
    } else {
	catch {set fqdn [exec hostname]}
    }
    if { [string first "." $fqdn] == -1 } {
	catch {set fqdn "${fqdn}.[exec domainname]"}
    }
    return "
[endNotes]
\\end{document}
% Converted to LaTeX by $env(USER)@$fqdn at [exec date]
"
}

proc endNotes {} {
    withNode docroot descendant element NOTE withattval PLACE END {
	return "
\\cleardoublepage
\\begingroup
\\parskip 2ex
\\parindent 0pt
\\def\\enotesize{\\normalsize}
\\theendnotes
\\endgroup
"
    }
    return
}

proc definePageStyle {} {
    return
}

proc defineTitlePageStyle {} {
    return
}

proc figPlacement {} {
    return "!htbp"
}

proc insertGraphicFile { fileName } {
    return [includegraphics $fileName EPS eps]
}

proc includegraphics { fileName args } {
    debug "includegraphics: $fileName $args"
    set listOfFiles $fileName
    foreach extension $args {
	append listOfFiles " ${fileName}.${extension}"
    }
    set fileName [cutGraphicExtensionOff $fileName]
    foreach extension $args {
	append listOfFiles " ${fileName}.${extension}"
    }
    debug "listOfFiles: $listOfFiles"
    foreach realName $listOfFiles {
	debug "looking for $realName"
	if { [file readable $realName] } {
	    return [reallyInsertGraphicFile $realName]
	}
    }
    puts stderr "***WARNING: No such file: $fileName\n"
    return
}

proc reallyInsertGraphicFile { fileName } {
    return "\n\\IfFileExists{$fileName}{\\includegraphics*{$fileName}}{}\n"
}
