# $Id: locale.koi8-r.tcl,v 1.32 1999/11/21 06:23:55 tobotras Exp $

debug "Start of koi8-r locale..."

substitution localizeSubst {
    {prev} {}
    {next} {}
    {abstract} {}
    {contents} {}
    {thebibliography} { }
    {appendices} {}
    {LaTeX_preamble} {
	%-- Using T2 requires this:
	\\usepackage\[koi8-r\]\{inputenc\}
        \\usepackage\{mathtext\}
	%--
	
	\\usepackage\[english,russian\]\{babel\}

	%-- Not using T2 requires this:
	%\\makeatletter
	%\\renewcommand\\listoffigures\{%
	%    \\section*\{ 
	%	\\@mkboth\{ \}\{ \}\}%
	%    \\@starttoc\{lof\}%
	%\}

	%\\renewcommand\\listoftables\{%
	%    \\section*\{ 
	%	\\@mkboth\{ \}\{ \}\}%
	%    \\@starttoc\{lot\}%
	%\}
	%\\makeatother
	%--
	
	\\usepackage\{footnpag,indentfirst,amstext\}

	\\makeatletter

	%\\let\\@alph\\@asbuk
	%\\let\\@Alph\\@Asbuk

	\\renewcommand\\appendix{\\par
	    \\setcounter{section}{0}%
	    \\setcounter{subsection}{0}%
	    \\gdef\\thesection{\\@Asbuk\\c@section}}

	\\makeatother

	\\renewcommand{\\theenumii}{\\asbuk{enumii}}

    }
    {pageBreak} {}
    {cyr} {\\cyr }
    {endfoot} { }
    {endhead} {\\ldots }
    {continue_caption} {}
    {lq} {<<}
    {rq} {>>}
    {producedby} {}
    {modified} { :}
    {texMdash} {~--- }
    {rtfMdash} {$rtfSpecial(EnSpace)$rtfSpecial(EmDash) }
    {Heading} {}
    {TOC} {}
    {index} { }
    {manName} {}
    {manSynopsis} {}
    {manDescription} {}
    {manBugs} {}
    {manNotes} {}
    {manOptions} {}
    {endNotes} {}
    {noteHead} {:}
    {listoftables} { }
    {listoffigures} { }
}

environment TeXlanguage current "ru"

proc localize { smth } {

    switch "$smth" {
	"lq" {
	    if { [TeXlanguage get current] == "ru" } {
		return [localizeSubst "lq"]
	    } else {
		return "``"
	    }
	}
	"rq" {
	    if { [TeXlanguage get current] == "ru" } {
		return [localizeSubst "rq"]
	    } else {
		return "''"
	    }
	}
	default {
	    # All globals referenced from localizeSubst should go here
	    global rtfSpecial
	    return "[subst [localizeSubst $smth]]"
	}
    }
}

proc appendixLetter { number } {
    return [string index "" [expr $number - 1]]
}

proc listLetter { number } {
    return [string index "" [expr $number - 1]]
}

# RTF only

proc tabTitle { number } {
    return " ${number}:"
}

proc figTitle { number } {
    return ". ${number}:"
}

proc appendixPrefix { number } {
    return " [appendixLetter $number]"
}

# LaTeX only

proc openLang { lang } {
    TeXlanguage save
    TeXlanguage set current $lang

    switch $lang {
	"ru" {
	    # This is default in koi8-r locale
	    return
	} 
	"en" {
	    return "\{\\selectlanguage{english}"
	}
    }
    return "\{"
}

proc closeLang {} {
    TeXlanguage restore
#    return "\\selectlanguage{russian}"
    return "\}"
}

# HTML only

proc SeeTocAndText { tocURL bodyURL } {
    return "

	    <A HREF=\"${tocURL}\"></a>   
	<A HREF=\"${bodyURL}\"> </a>.
"
}

# SGML only

proc localeDivName { gi } {
    switch -regexp $gi {
	"(DIV)|(DIV[01])" {
	    return ""
	}
	"DIV2" {
	    return ""
	}
	"DIV3" {
	    return ""
	}
	default {
	    return ""
	}
    }
}

debug "End of koi8-r locale..."

