# $Id: stylesheet.tcl,v 1.22 1999/06/02 09:20:13 tobotras Exp $

puts -nonewline stderr "Loading stylesheet..."

rtf:documentFormat {
    TopMargin		0.5in
    BottomMargin	0.5in
    FootnoteNumbering	Arabic
    FootnoteRestart	AtPage
    FootnotePlacement	PageBottom
}

# ------------------ rules

rtf:ruleStyle style_Border {
    Thickness 0.75pt
    Margin 1.5mm
}

rtf:ruleStyle rule_Table_Col_Sep {
    Thickness 0.75pt
}

rtf:ruleStyle rule_Table_Row_Sep {
    Margin 1mm
    Thickness 0.75pt
}

rtf:ruleStyle rule_Table_Frame {
    Thickness 1.5pt
}

# -------------------- sections

rtf:sectStyle style_Front "Front section" {
    LeftMargin		1cm
    RightMargin		1cm
    HasTitlePage	1
    SectionBreak	None
}

rtf:sectStyle style_Body "Body section" {
    LeftMargin		3cm
    RightMargin		2cm
}

rtf:sectStyle style_Index "Index section" {
    LeftMargin		3cm
    RightMargin		2cm
}

rtf:sectStyle style_Landscape "Landscape section" {
    LeftMargin		3cm
    RightMargin		2cm
    Landscape		1
}

# -------------------- paragraphs

rtf:paraStyle style_DocTitle "Document title" {
    Font sans
    FontSize 18pt
    Bold 1
    Quadding Center
    SpaceBefore 5cm
    SpaceAfter 2cm
}

rtf:paraStyle style_DocAuthor "Document author" {
    Font roman
    FontSize 16pt
    Quadding Center
    SpaceBefore 1cm
    SpaceAfter 2cm
}

rtf:paraStyle style_DocDate "Document date" -basedon style_DocAuthor {
    FontSize 14pt
    SpaceBefore 5cm
}

rtf:paraStyle style_Default "Normal" {
    FontSize 12pt
    Font roman
    SpaceBefore 6pt
    LeftIndent 0pt
    FirstIndent 0mm
    LineSpacing 18pt
    Quadding Left
}

rtf:paraStyle style_Default_Bold "Bold" -basedon style_Default {
    Bold 1
}

rtf:paraStyle style_Default_Left "Ragged left" -basedon style_Default {
    Quadding Right
}

rtf:paraStyle style_Default_Right "Ragged right" -basedon style_Default {
    Quadding Left
}

rtf:paraStyle style_Default_Center "Centered" -basedon style_Default {
    Quadding Center
}

rtf:paraStyle style_Default_Justify "Justified" -basedon style_Default {
    Quadding Justify
}

rtf:paraStyle default "Normal" -basedon style_Default {
}

rtf:paraStyle style_Abstract "Abstract" -basedon style_Default {
    Italic 1
}

rtf:paraStyle style_Picture "Figure" -basedon style_Default_Center {
    SpaceBefore 8mm
    SpaceAfter 5mm
    KeepWithNext 1
}

rtf:paraStyle style_Picture_Title "Figure title" -basedon style_Default {
    Bold 1
    LeftIndent 2cm
    RightIndent 2cm
    Quadding Center
}

rtf:paraStyle style_Default_Noindent "Noindent" -basedon style_Default {
    FirstIndent 0pt
}

rtf:paraStyle style_Verbatim "Verbatim" -basedon style_Default {
    Font mono
    Bold 1
    FirstIndent 0pt
}

rtf:paraStyle style_Para_Code "Sample" -basedon style_Default {
    SpaceBefore 6mm
    SpaceAfter 6mm
}

rtf:paraStyle style_Para_Note "Note interline" -basedon style_Default {
    SpaceBefore 6mm
    SpaceAfter 6mm
    Italic 1
}

rtf:paraStyle style_Code "Code sample" -basedon style_Default {
    Font mono
    FirstIndent 0mm
}

rtf:paraStyle style_Heading_1 "[localize Heading] 1" -basedon style_Default {
    Font sans
    FontSize 16pt
    Bold 1
    Quadding Left
    FirstIndent 0mm
    SpaceBefore 12pt
    SpaceAfter 3pt
}

rtf:paraStyle style_Heading_2 "[localize Heading] 2" -basedon style_Heading_1 {
    FontSize 14pt
}

rtf:paraStyle style_Heading_3 "[localize Heading] 3" -basedon style_Heading_2 {
    Bold 0
}

rtf:paraStyle style_Heading_4 "[localize Heading] 4" -basedon style_Heading_3 {
    Italic 1
}

rtf:paraStyle style_Heading_5 "[localize Heading] 5" -basedon style_Heading_4 {
    FontSize 12pt
}

rtf:paraStyle style_Heading_6 "[localize Heading] 6" -basedon style_Heading_5 { }

rtf:paraStyle style_Heading_7 "[localize Heading] 7" -basedon style_Heading_6 { }

rtf:paraStyle style_Display_Title "Head & Title" -basedon style_Heading_2 { }

rtf:tabStops tocTabs {
    "5in Align Left Leaders Dot"
}

rtf:paraStyle style_TOC_0 "[localize TOC] 1" { 
    Bold	1
    Font	sans
    FontSize	16pt
    LeftIndent	1cm
    TabStops	tocTabs
}

rtf:paraStyle style_TOC_1 "[localize TOC] 2" -basedon style_TOC_0 { 
    LeftIndent	2cm
    FontSize	14pt
}
rtf:paraStyle style_TOC_2 "[localize TOC] 3" -basedon style_TOC_1 { 
    LeftIndent	3cm
    FontSize	12pt
}
rtf:paraStyle style_TOC_3 "[localize TOC] 4" -basedon style_TOC_2 { 
    LeftIndent	3.5cm
    FontSize	10pt
}

rtf:paraStyle style_TOC_4 "[localize TOC] 5" -basedon style_Heading_5 { 
    LeftIndent	3.5cm
    Italic	1
    Bold	0
}

rtf:paraStyle style_List_Heading_1 "List heading 1" -basedon style_Default {
    Bold 1
    FontSize 14pt
    LeftIndent 10mm
    FirstIndent 0pt
    SpaceAfter 0pt
}

rtf:paraStyle style_List_Heading_2 "List heading 2" -basedon style_List_Heading_1 {
    LeftIndent 15mm
}

rtf:paraStyle style_List_Heading_3 "List heading 3" -basedon style_List_Heading_2 {
    LeftIndent 20mm
}

rtf:paraStyle style_List_Heading_4 "List heading 4" -basedon style_List_Heading_3 {
}

rtf:paraStyle style_List_Item_1 "List item 1" -basedon style_Default {
    FirstIndent -5mm
    LeftIndent 15mm
    SpaceAfter 0pt
    SpaceBefore 0pt
}

rtf:paraStyle style_List_Item_2 "List item 2" -basedon style_List_Item_1 {
    LeftIndent 20mm
}

rtf:paraStyle style_List_Item_3 "List item 3" -basedon style_List_Item_2 {
    LeftIndent 25mm
}

rtf:paraStyle style_List_Item_4 "List item 4" -basedon style_List_Item_3 {
    LeftIndent 30mm
}

rtf:paraStyle style_Para_Quote "Quotation" -basedon style_Default {
    LeftIndent 1cm
    RightIndent 1cm
    FirstIndent 0pt
}

rtf:paraStyle style_Stage "Stage" -basedon style_Default {
    LeftIndent 1cm
    RightIndent 1cm
    FirstIndent 0pt
    Quadding Center
    FontSize 10pt
}

rtf:paraStyle style_Address "Address" -basedon style_Para_Quote {
}

rtf:paraStyle style_Colophon "Colophon" -basedon style_Default {
    FontSize 10pt
}

rtf:paraStyle style_List_Label_1 "Item label 1" -basedon style_List_Item_1 {
    Bold 1
    Font sans
}

rtf:paraStyle style_List_Label_2 "Item label 2" -basedon style_List_Item_2 {
    Bold 1
    Font sans
}

rtf:paraStyle style_List_Label_3 "Item label 3" -basedon style_List_Item_3 {
    Bold 1
    Font sans
}

rtf:paraStyle style_List_Label_4 "Item label 4" -basedon style_List_Item_4 {
    Bold 1
    Font sans
}

rtf:paraStyle style_Table_Head_Para "Table header" -basedon style_Default {
    LeftIndent 2cm
    RightIndent 2cm
    SpaceAfter 0pt
    SpaceBefore 0pt
    KeepWithNext 1
}

rtf:paraStyle style_Table_Cell_Left "Cell left" -basedon style_Default {
    SpaceBefore 0pt
    SpaceAfter 6pt
    FirstIndent 0pt
    LeftIndent 1mm
    RightIndent 1mm
    Quadding Left
}

rtf:paraStyle style_Table_Cell_Right "Cell right" -basedon style_Table_Cell_Left {
    Quadding Right
}

rtf:paraStyle style_Table_Cell_Center "Cell center" -basedon style_Table_Cell_Left {
    Quadding Center
}

rtf:paraStyle style_Table_Cell_Justify "Cell justify" -basedon style_Table_Cell_Left {
    Quadding Justify
}

rtf:paraStyle style_PageFooter "Page footer" -basedon style_Default {
    Quadding Center
}

rtf:paraStyle style_PageHeader "Page header" -basedon style_Default {
    Quadding Center
}

rtf:paraStyle style_Epigraph "Epigraph" -basedon style_Default {
    LeftIndent 4in
}

# ------------------ chars

rtf:charStyle style_Plain "Plain" {
    Font roman
    FontSize 12pt
    Bold 0
    Italic 0
}

rtf:charStyle style_Table_Head_Phrase "Table head" -basedon style_Plain {
    Bold 1
    Italic 1
}

rtf:charStyle style_Typewriter "Typewriter" -basedon style_Plain {
    Font mono
}

rtf:charStyle style_Italic "Italic" -basedon style_Plain {
    Italic 1
}

rtf:charStyle style_Sans "Sans Serif" -basedon style_Plain {
    Font sans
}

rtf:charStyle style_Xref "X-reference" -basedon style_Plain {
    Italic 1
}

rtf:charStyle style_Slanted "Slanted" -basedon style_Plain {
    Italic 1
}

rtf:charStyle style_Bold "Bold" -basedon style_Plain {
    Bold 1
}

rtf:charStyle style_Plain_Bold "Bold" -basedon style_Bold {
}

rtf:charStyle style_Italic_Bold "Bold italic" -basedon style_Italic {
    Bold 1
}

rtf:charStyle style_Slanted_Bold "Bold slanted" -basedon style_Slanted {
    Bold 1
}

rtf:charStyle style_Typewriter_Bold "Bold typewriter" -basedon style_Typewriter {
    Bold 1
}

rtf:charStyle style_Ident "Identifier" -basedon style_Plain {
    Bold 1
}

rtf:charStyle style_Term "Term" -basedon style_Plain {
    Bold 1
}

rtf:charStyle style_Mentioned "Mentioned" -basedon style_Plain {
    Italic 1
}

rtf:charStyle style_Phrase_Code "Inline sample" -basedon style_Plain {
    Font mono
    Bold 1
}

rtf:charStyle style_Speaker "Speaker" -basedon style_Plain {
    LetterSpace 3pt
}

rtf:charStyle style_StageInText "Stage directions" -basedon style_Plain {
    Italic 1
}

rtf:charStyle style_Gi "GI style" -basedon style_Plain {
    Font mono
    Bold 1
}

rtf:charStyle style_Title "Title style" -basedon style_Plain {
    Italic 1
}

rtf:charStyle style_Phrase_Quote "Quote" -basedon style_Plain {
}

rtf:charStyle style_Phrase_Note "Note" -basedon style_Plain {
    Font sans
}

rtf:charStyle style_FootnoteNumber "Footnote number" -basedon style_Phrase_Note {
    FontSize 6pt
    Superscript 1
}

rtf:charStyle style_Footnote "Footnote" -basedon style_Phrase_Note {
    FontSize 10pt
}

rtf:charStyle style_Keyword "Keyword" -basedon style_Plain {
    Font mono
    Bold 1
}

rtf:charStyle style_Foreign "Foreign" -basedon style_Plain {
}

rtf:charStyle style_Underline "Underline" -basedon style_Plain {
    Underline Single
}

rtf:charStyle style_Superscript "Superscript" -basedon style_Plain {
    Superscript 1
}

rtf:charStyle style_Subscript "Subscript" -basedon style_Plain {
    Subscript 1
}

rtf:charStyle style_Smallcaps "SmallCaps" -basedon style_Plain {
    SmallCaps 1
}

rtf:charStyle style_Underline_Bold "Bold Underline" -basedon style_Underline {
    Bold 1
}

rtf:charStyle style_Superscript_Bold "Bold Superscript" -basedon style_Superscript {
    Bold 1
}

rtf:charStyle style_Subscript_Bold "Bold Subscript" -basedon style_Subscript {
    Bold 1
}

rtf:charStyle style_Smallcaps_Bold "Bold SmallCaps" -basedon style_Smallcaps {
    Bold 1
}

# Fallbacks for unknown styles
rtf:charStyle style_Unknown "Undefined" -basedon style_Plain {
    Bold 1
    Italic 1
}

rtf:paraStyle style_Unknown "Undefined" -basedon style_Default {
    Bold 1
    Italic 1
}

puts stderr ""
